///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishImage.hh"
#include "wishDistanceMapFilter.hh"
#include "wishGaussianSmoothingFilter.hh"
#include "wishGaussianGradientMagnitudeFilter.hh"
#include "wishGLLiveMarching.hh"
#include "wishSigmoidFilter.hh"

using namespace wish;

int main(int argc, char **argv) { 
  
  if(argc <= 1) {
    cerr << "Welcome!\n";
    cerr << "Left mouse sets points, space deletes them.\n";
    cerr << "Use F1, F2, and F3 to view original, cost, or the fast marching images.\n\n";
    cerr << "This image was selected by default since you did not provide any filename(s). Next time, try \"live_marching.exe [original_image cost_image]\"" << endl;   
    



    ImageObject *in = ImageObject::createFromFile( string( getenv("WISH_ROOT") )+"/data/mri_ventricles.vtk" );
    
    in->printSelf();
    
    if( !in ) {
      cerr << "Error reading image" << endl;
      exit(1);
    }
    
    Image8 *tmp = dynamic_cast<Image8*>(in);
    Image8 im8;    
    int d = tmp->depth();
    if( d>1 ) 
      im8 = tmp->extractSlice(d/2);
    else
      im8 = *tmp;
    
    GaussianGradientMagnitudeFilter<Image8, Image8> ggmf;
    ggmf.setInput( &im8 );
    ggmf.setSigma( 1.0 );
    ggmf.execute();
    
    SigmoidFilter<Image8, Image8> sigmoid;
    sigmoid.setInput( ggmf.getOutput() );
    sigmoid.setAlpha( 2.0 );
    sigmoid.setBeta( 5.0 );
    sigmoid.setMax( 0 );
    sigmoid.setMin( 255 );
    sigmoid.execute();
    
    GLLiveMarching *lm = GLLiveMarching::getInstance();
    lm -> setCostInput( sigmoid.getOutput() );
    lm -> setInput( &im8 );
    lm -> render();
  }
  else if(argc==2) {
    ImageObject *in = ImageObject::createFromFile(string(argv[1]));
    
    in->printSelf();
    
    if( !in ) {
      cerr << "Error reading image" << endl;
      exit(1);
    }
    
    Image8 *tmp = dynamic_cast<Image8*>(in);
    Image8 im8;    
    int d = tmp->depth();
    if( d>1 ) 
      im8 = tmp->extractSlice(d/2);
    else
      im8 = *tmp;
    
    GaussianGradientMagnitudeFilter<Image8, Image8> ggmf;
    ggmf.setInput( &im8 );
    ggmf.setSigma( 2.0 );
    ggmf.execute();
    
    SigmoidFilter<Image8, Image8> sigmoid;
    sigmoid.setInput( ggmf.getOutput() );
    sigmoid.setAlpha( 2.0 );
    sigmoid.setBeta( 5.0 );
    sigmoid.setMax( 0 );
    sigmoid.setMin( 255 );
    sigmoid.execute();
    
    GLLiveMarching *lm = GLLiveMarching::getInstance();
    lm -> setCostInput( sigmoid.getOutput() );
    lm -> setInput( &im8 );
    lm -> render();
  }

  else if(argc==3) {
    ImageObject *in = ImageObject::createFromFile(string(argv[1]));
    ImageObject *cost = ImageObject::createFromFile(string(argv[2]));
    
    in->printSelf();
    cost->printSelf();
    
    if( !in || !cost ) {
      cerr << "Error reading images" << endl;
      exit(1);
    }
    GLLiveMarching *lm = GLLiveMarching::getInstance();
    lm -> setCostInput(cost);
    lm -> setInput(in);
    lm -> render();
  }
  else {
    cerr << "Usage: " << argv[0] << " [original_image cost_image]" << endl;
    exit(0);
  }
  return 0;
}
