///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishdemo.hh"

#include "wishFuzzyConnectednessFilter.hh"

// convert from window coordinates to image coordinates
Vec3f windowToImage(int x, int y) {
  GLint viewport[4];
  GLdouble modelview[16];
  GLdouble projection[16];
  GLfloat winX, winY, winZ;
  GLdouble posX, posY, posZ;
  
  glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
  glGetDoublev( GL_PROJECTION_MATRIX, projection );
  glGetIntegerv( GL_VIEWPORT, viewport );
  
  winX = (float)x;
  winY = (float)viewport[3] - (float)y;
  glReadPixels( x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );
  
  gluUnProject( winX, winY, winZ, 
		modelview, projection, viewport, 
		&posX, &posY, &posZ);
  
  Vec3f p( (float)posX, (float)posY, (float)posZ );
  
  p = im->getWorldToImageMatrix()*p;
  p.z = (float)slice;
  
  return p;
}

// display callback
void display() {
  
  Vec3f te = im->getTotalExtent();
  
  // clear buffer
  glClear(GL_COLOR_BUFFER_BIT);
  
  // #################### MPR ####################
  
  // viewport
  glViewport(winx+winw2, winy, winw2, winh);
  //glViewport(winx, winy, winw2, winh);

  // perspective projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  
  // field of view angle, aspect ratio, near, far
  gluPerspective(45.0f, aspect, 10, 5000); 
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  // clear depth buffer
  glClear(GL_DEPTH_BUFFER_BIT);

  // trackball rotation
  float eyedist = float( (camera->getEye() - camera->getAt()).norm() );
  glTranslatef(0,0,-eyedist);
  trackball->applyRotation();
  glTranslatef(0,0,eyedist);
  
  // view transformation
  camera -> viewTransformation();
  
  // scaling
  glScalef(scl, scl, scl);
  
  // render volume
  //slice_viewer->render( true, true, true );
  volume_renderer->render();
  
  // render outline
  glPushAttrib( GL_POLYGON_BIT | GL_LINE_BIT | GL_COLOR_BUFFER_BIT );
  glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
  glLineWidth( 2.0f );
  glEnable(GL_LINE_SMOOTH);
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glScalef( te.x, te.y, te.z );
  glutWireCube( 1.01f );
  glPopAttrib();

  // #################### AXIAL (Z) VIEW ####################
  
  // viewport
  glViewport(winx, winy, winw2, winh);
  //glViewport(winx+winw2, winy, winw2, winh);

  // ortho projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  if( aspect<1.0f ) 
    glOrtho( -.5f*te.x, .5f*te.y, 
	     -.5f*te.x/aspect, .5f*te.y/aspect, 
	     0, 10000 );  
  else
    glOrtho( -.5f*te.x*aspect, .5f*te.y*aspect, 
	     -.5f*te.x, .5f*te.y, 
	     0, 10000 );
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  // clear depth buffer
  glClear(GL_DEPTH_BUFFER_BIT);
  
  // view transformation
  camera2 -> viewTransformation();
  
  // translation
  glTranslatef(slice_translation.x, slice_translation.y, 0.0f);
  
  // scaling
  glScalef(slice_zoom, slice_zoom, 1.0f);
  
  // slice viewer
  slice_viewer->render();

  // infotext
  infotext->setPosition( .02f, .03f );
  infotext->setString( "File: " + filename );
  infotext->render();
  
  infotext->setPosition( .02f, .06f );
  infotext->setString( "Dimensions: (" + 
		       util::typeToString(im->width()) + ", " +
		       util::typeToString(im->height()) + ", " +
		       util::typeToString(im->depth()) + ")");
  infotext->render();
  
  int mx = util::round( mousepos.x );
  int my = util::round( mousepos.y );
  
  infotext->setPosition( .02f, .09f );
  infotext->setString( "Position:   (" + 
		       util::typeToString( mx, 0, 3 ) +
		       ", " + 
		       util::typeToString( my, 0, 3 ) +
		       ", " +
		       util::typeToString( slice, 0, 3 ) +
		       ")" );
  infotext->render();
  
  string valstring = "-";
  if( im->isInside(mx,my,slice) )
    valstring = util::typeToString( im->getSafeAsDouble(mx,my,slice), 0, 3 );
  
  infotext->setPosition( .02f, .12f );
  infotext->setString( "Value:       " + valstring );
  infotext->render();

  valstring = "-";
  if( im->isInside(mx,my,slice) )
    valstring = 
      util::typeToString( im2->getSafeNormalized(mx,my,slice), 3, 4 );
  
  infotext->setPosition( .02f, .15f );
  infotext->setString( "FC value:    " + valstring );
  infotext->render();
  
  // ###############

  // flush and swap buffers
  glFlush();
  glutSwapBuffers();
}

// idle callback
void idle() {
  glutSetWindow(mainwin);
  glutPostRedisplay();
}

// keyboard callback
void keyboard(unsigned char key, int x, int y) {
  
  x; y;
  
  switch(key) {
    
    // reset rotation, scaling, contrast, and brightness
  case 'r':
    trackball->reset();
    yangle = 0.0f;
    scl = init_scl;
    wl_width = 1.0f;
    wl_level = 0.5f;
    slice_viewer->setWL(wl_width, wl_level);
    volume_renderer->setWL(wl_width, wl_level);
    static_cast<Image16*>(im2)->reset( 0 );
    tex2->initialize();
    break;
    
  case 'q':
  case 'Q':
  case 27: // escape key
    exit(0);
    break;
  default:
    break;
  }
  glutPostRedisplay();
}

void specialKeys(int key, int x, int y) {
  switch(key) {
  case GLUT_KEY_UP:
    slice++;
    if(slice > im->depth()-1 ) slice = im->depth()-1;
    slice_viewer->setZSlice(slice);
    break;
  case GLUT_KEY_DOWN:
    slice--;
    if(slice < 0 ) slice = 0;
    slice_viewer->setZSlice(slice);
    break;
  case GLUT_KEY_LEFT:
    break;
  case GLUT_KEY_RIGHT:
    break;
  }
  glutPostRedisplay();
}

// mouse callback
void mouse(int btn, int state, int x, int y) {
  
  if( btn==GLUT_LEFT_BUTTON ) {
    if( state==GLUT_UP ) {
      trackball->deactivate();
    }
    if( winw2 <= x && x <= fullw ) { // right vp
      if( state==GLUT_DOWN ) {
	trackball->activate();
	trackball->grab(x-winw2,y,winw2,winh);
      }
    }
    
    else {
      Vec3i seed = mousepos;      
      vector<Vec3i> seeds;
      for( int z=-2; z<=2; z++ ) {
	for( int y=-2; y<=2; y++ ) {
	  for( int x=-2; x<=2; x++ ) {
	    Vec3i tmp = Vec3i(x,y,z);
	    if( tmp*tmp<=2.0*2.0 ) {
	      tmp = seed + tmp;
	      if( im->isInside(tmp) )
		seeds.push_back( tmp );
	    }
	  }
	}
      }
      
      if( im->isInside( mousepos ) ) {
	if( im->getDataType()=="uint8" ) {
	  FuzzyConnectednessFilter<Image8,Image16> FC;
	  FC.setInput( static_cast<Image8*>( im ) );
	  Image16 tmp;	  
	  FC.setOutput( &tmp );
	  FC.setEarlyThreshold( 0.5 );
	  FC.setSeeds( seeds );
	  FC.execute();

	  Image16 *fcoutput = static_cast<Image16*>( im2 );
	  for(int i=0; i<fcoutput->totalSize(); i++)
	    fcoutput->at(i) = util::maxOf( fcoutput->at(i), tmp[i] );
	}
	else if( im->getDataType()=="uint16" ) {
	  FuzzyConnectednessFilter<Image16,Image16> FC;
	  FC.setInput( static_cast<Image16*>( im ) );
	  Image16 tmp;	  
	  FC.setOutput( &tmp );
	  FC.setEarlyThreshold( 0.5 );
	  FC.setSeeds( seeds );
	  FC.execute();
	  
	  Image16 *fcoutput = static_cast<Image16*>( im2 );
	  for(int i=0; i<fcoutput->totalSize(); i++)
	    fcoutput->at(i) = util::maxOf( fcoutput->at(i), tmp[i] );
	}
	
	tex2->initialize();
	
      }
    }

  }
  
  else if( btn==GLUT_MIDDLE_BUTTON ) {
    if( state == GLUT_DOWN ) {
      mouse_wl = true;
      wl_grabx = x;
      wl_graby = y;
    }
    else {
      mouse_wl = false;
    }
  }
  
  else if( btn==GLUT_RIGHT_BUTTON ) {
    if( winw2 <= x && x <= fullw ) { // right vp
      if( state==GLUT_DOWN ) {
	mouse_scale = true;
	scale_graby = y;
      }
    }
    if( state==GLUT_UP ) {
      mouse_scale = false;
    } 
  }
  
  glutPostRedisplay();
}

// motion callback
void motion(int x, int y) {
  
  if( trackball->isActive() ) {
    trackball->move(x-winw2,y,winw2,winh);
  }
  
  if( mouse_wl ) {
    float dx = (x-wl_grabx)*0.01f;
    float dy = -(y-wl_graby)*0.01f;
    wl_width += dx;
    if( wl_width < -10.0f ) wl_width = -10.0f;
    if( wl_width > 10.0f ) wl_width = 10.0f;
    wl_level += dy;
    if( wl_level <= -1.0f ) wl_level = -1.0f;
    if( wl_level >= 1.0f ) wl_level = 1.0f;
    wl_grabx = x;
    wl_graby = y;
    slice_viewer->setWL(wl_width, wl_level);
    volume_renderer->setWL(wl_width, wl_level);
  }
  
  if( mouse_scale ) {
    float ds = -(float(y)-scale_graby);
    scl = scl+ds*0.01f;
    if( scl < 0.1f ) scl = 0.1f;
    if( scl > 10.0f ) scl = 10.0f;
    scale_graby = y;
  }
  
  glutPostRedisplay();  
}

// passive motion callback
void passiveMotion(int x, int y) {
  
  if( 0 <= x && x <= winw2 )
    mousepos = windowToImage(x,y);
  else
    mousepos = Vec3f(-1.0f, -1.0f, -1.0f);
  
  glutPostRedisplay();  
}

// menu callback
void menu(int id) {
  
  if(id == 99) 
    exit(0);
  glutPostRedisplay();
  
}

// reshape callback, perspective projection
void reshape(int w, int h) {
  
  glViewport( 0, w, 0, h );
  
  // update our variables
  fullw = w;
  fullh = h;
  winx = 0;
  winy = 0;
  winw = w;
  winw2 = winw/2;
  winh = h;
  aspect = float(winw2)/winh;
  
  // update slice viewer
  slice_viewer->reshape(winw2,winh);

}

// initialize
void initialize(int argc, char **argv) {
  
  // color to clear the frame buffer with
  glClearColor(0.1f, 0.1f, 0.1f, 1.0f);
  
  // enable z-buffering
  glEnable(GL_DEPTH_TEST);
  
  // trackball
  trackball = new Trackball;

  // camera
  camera = new Camera;
  camera -> setEye( Vec3f(0.0,0.0,400.0) );
  camera -> setAt( Vec3f(0,0,0) );
  camera -> setUp( Vec3f(0,1,0) );
  
  camera2 = new Camera;
  camera2 -> setEye( Vec3f(0.0,0.0,800.0) );
  camera2 -> setAt( Vec3f(0,0,0) );
  camera2 -> setUp( Vec3f(0,1,0) );
  
  // apply this before defining the lightsource
  // so that the lights position will be transformed by
  // the view matrix
  camera -> viewTransformation();
  
  if( argc == 2 ) {
    filename = string(argv[1]);
    filename2 = "";
  }
  else {
    cerr << "Usage: " << argv[0] << " imagefile" << endl;
    exit(0);
  }
  
  cerr << "Welcome! Try to click the slice in the left viewport. You will get a fuzzy segmentation (shown in red). The right viewport shows a MIP of the data.\n\n";
  cerr << "UI:\n\n";
  cerr << "   Left mouse button:   Seed (left vp) or Rotate (right vp)\n";
  cerr << "   Middle mouse button: Window/Level adjustment\n";
  cerr << "   Right mouse button:  Scale (right vp)\n";
  cerr << "   Arrow up/down:       Browse slices\n";
  cerr << "   Keyboard 'r':        Reset\n";
  cerr << "   Keyboard 'q'/Esc:    Quit\n\n";
  
  // read image
  im = ImageObject::createFromFile( filename );
  
  if( im==0 ) {
    cerr << "Error reading image" << endl;
    exit(1);
  }
    
  // create secondary image
  im2 = new Image16;
  im2->allocate( im->width(), im->height(), im->depth() );
  im2->setSpacing( im->getSpacing() );
  
  // textures
  tex = new Texture(im);  
  tex2 = new Texture(im2);  
  
  // slice viewer
  slice_viewer = new SliceViewer;
  slice_viewer->addTexture( tex );  
  slice_viewer->addTexture( tex2 );
  slice_viewer->initialize( shader_ok );
  slice = im->getDepth()/2;
  slice_viewer->setZSlice( im->getDepth()/2 );
  slice_viewer->setYSlice( im->getHeight()/2 );
  slice_viewer->setXSlice( im->getWidth()/2 );

  // slice viewer
  volume_renderer = new GPUVolumeRenderer;
  volume_renderer->addTexture( tex );  
  volume_renderer->addTexture( tex2 );
  volume_renderer->setBlend( 0.2f );
  volume_renderer->setRayStep( 0.005f );
  volume_renderer->initialize( shader_ok );
  
  // text
  infotext = new Text;
}

// main function
int main(int argc, char **argv) {
  
  // initialize glut
  // create rendering window with double framebuffers and depth buffer
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(winw, winh);
  mainwin = glutCreateWindow("..:: WISH ::..");
  
  //initialize glew
  //glewExperimental = GL_TRUE;
  GLenum err = glewInit();
  if( err != GLEW_OK ) {
    cerr << "GLEW initialization failed:" << endl
	 << glewGetErrorString(err) << endl;
    exit(1);
  }
  
  //check for extensions
  cerr << "Checking GL extensions" << endl;
  shader_ok = 
    checkExtension( EXT_texture3D ) && 
    checkExtension( ARB_shader_objects ) &&
    checkExtension( ARB_vertex_shader ) &&
    checkExtension( ARB_fragment_shader );
  if( !shader_ok )
    cerr << "Shaders are disabled" << endl;
  
  // initialize
  initialize(argc, argv);
  
  // specify callback functions
  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutKeyboardFunc(keyboard);
  glutSpecialFunc(specialKeys);
  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  glutPassiveMotionFunc(passiveMotion);
  glutIdleFunc(idle);
  
  // start the glut event loop
  glutMainLoop();
  
  // clean up
  delete slice_viewer;
  delete camera;
  delete trackball;
  delete im;
  delete im2;
  delete tex;
  delete tex2;
  
  return 0;
}
