///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISHDEMO_HH_
#define _WISHDEMO_HH_

#if defined (__WIN32__) || defined (WIN32)
#include <windows.h>
#endif

#include <GL/glew.h> // should always be included first!!!
#include <GL/glut.h>

#include "FreeImage.h"

#include "wishImage.hh"
#include "wishGLTexture.hh"
#include "wishGLText.hh"
#include "wishGLCamera.hh"
#include "wishGLTrackball.hh"
#include "wishSliceViewer.hh"
#include "wishGPUVolumeRenderer.hh"
#include "wishShaderUtilities.hh"
#include "wishFuzzyConnectednessFilter.hh"

using namespace wish;
using namespace std;

// main window
int mainwin;

// shader flag
bool shader_ok = true;

// camera
Camera *camera, *camera2;

// trackball
Trackball *trackball;
float yangle = 0.0;

// mouse pos
Vec3f mousepos = Vec3f( 0.0f, 0.0f, 0.0f );

// text
Text *infotext;
string infostring;

// the image
string filename;
string filename2;
string savename;
ImageObject *im;
ImageObject *im2;
Texture *tex;
Texture *tex2;
float zspacing = 3.0f;
float blend = 0.0f;

// slice viewer
SliceViewer *slice_viewer;
GPUVolumeRenderer *volume_renderer;

// slice translation
bool slice_pan=false;
Vec2f slice_translation;
Vec2f slice_pos;
Vec2f slice_grab;

// slice scaling 
float slice_zoom = 1.0f;
bool mouse_slice_zoom = false;
int slice_zoom_graby;

// slice
int slice=0;

// step
float init_ray_step = 0.02f;
float ray_step = init_ray_step;

// window dimensions
int fullw = 1024;
int fullh = 512;
int winw = 1024;
int winw2 = 512;
int winh = 512;
int winx = 0;
int winy = 0;
float aspect = 1.0;

// window/level
float wl_width = 1.0;
float wl_level = 0.5;
bool mouse_wl = false;
int wl_grabx;
int wl_graby;

// scaling 
float init_scl = 1.0f;
float scl = init_scl;
bool mouse_scale = false;
int scale_graby;

// convert from window coordinates to image coordinates for the slicer
Vec3f windowToImage(int x, int y);

// display callback
void display();

// idle callback
void idle();

// keyboard callback
void keyboard(unsigned char key, int x, int y);

void specialKeys(int key, int x, int y);

// mouse callback
void mouse(int btn, int state, int x, int y);

// motion callback
void motion(int x, int y);

// menu callback
void menu(int id);

// reshape callback, perspective projection
void reshape(int w, int h);

// initialize
void initialize(int argc, char **argv);

#endif
