///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

uniform sampler3D volumeTexture;
uniform sampler3D volumeTexture2;

uniform float rayStep;
uniform float wl_width;
uniform float wl_level;
uniform float blend;

varying vec3 raystart;
varying vec3 raydir;

float eps = 1e-6;

float windowLevel(float arg, float wlw, float wll) {
  return (arg-wll)/wlw + 0.5;
}

float rayexit(vec3 r0, vec3 dir) {
  vec3 q = 1.0/dir;
  vec3 t0 = -r0*q;
  vec3 tmax = max(t0,q+t0);
  return min(tmax.x,min(tmax.y,tmax.z));
}

void main() {
  
  // specify ray
  vec4 dir = vec4(normalize(raydir),-1.0);
  float l = rayexit(raystart, dir.xyz);
  dir *= rayStep;
  
  // maximum intensity projection
  vec4 col = vec4(0,0,0,0);
  vec4 raypos = vec4(raystart,l);
  while( raypos.w>=0.0 ) {
    vec4 col1 = texture3D(volumeTexture, raypos);
    vec4 col2 = texture3D(volumeTexture2, raypos);
    vec4 tmp = col1;
    if( col2.r > 0 )
      tmp = vec4(col2.r, 0, 0, 1);
    if( tmp.r>col.r ) 
      col = tmp;
    
    raypos += dir;
  }
  
  // set the computed color
  gl_FragColor = col;
  
}
