///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishCompareSegmentations.hh"

namespace wish {
  
  void CompareSegmentations::process() {
    
    Ctd = 0;
    Ctdc = 0;
    Cd = 0;
    Cu = 0;
    CTP = 0;
    CFP = 0;
    CTN = 0;
    CFN = 0;
    
    // fuzzy
    if( treat_as_fuzzy ) {
      
      for(int i=0; i<im_Ctd->totalSize(); i++) {
	double _Ctd, _Ctdc, _Cd, _Cu, _CTP, _CFP, _CTN, _CFN;
	
	double val_td = im_Ctd->getNormalizedFromArray(i);
	double val_d =  im_Cd->getNormalizedFromArray(i);
	
	_Ctd  = val_td;
	_Cd   = val_d;
	_CFN  = util::maxOf( _Ctd - _Cd, 0.0 );
	_CFP  = util::maxOf( _Cd - _Ctd, 0.0 );
	_CTP  = util::minOf( _Cd, _Ctd );
	_CTN  = 1.0 - _CFP;
	_Cu   = util::maxOf( _Cd, _Ctd );
	_Ctdc = 1.0 - _Ctd;
	
	Ctd  += _Ctd;
	Cd   += _Cd;
	CFN  += _CFN;
	CFP  += _CFP;
	CTP  += _CTP;
	CTN  += _CTP;
	Cu   += _Cu;
	Ctdc += _Ctdc;
      }
    } 
    
    // crisp
    else {
      
      for(int i=0; i<im_Ctd->totalSize(); i++) {
	double _Ctd, _Ctdc, _Cd, _Cu, _CTP, _CFP, _CTN, _CFN;
	
	double val_td = im_Ctd->getAsDoubleFromArray(i);
	double val_d =  im_Cd->getAsDoubleFromArray(i);
	
	_Ctd  = (val_td>0) ? 1.0:0.0; // crisp
	_Cd   = (val_d>0) ? 1.0:0.0;  // crisp
	_CFN  = util::maxOf( _Ctd - _Cd, 0.0 );
	_CFP  = util::maxOf( _Cd - _Ctd, 0.0 );
	_CTP  = util::minOf( _Cd, _Ctd );
	_CTN  = 1.0 - _CFP;
	_Cu   = util::maxOf( _Cd, _Ctd );
	_Ctdc = 1.0 - _Ctd;
	
	Ctd  += _Ctd;
	Cd   += _Cd;
	CFN  += _CFN;
	CFP  += _CFP;
	CTP  += _CTP;
	CTN  += _CTP;
	Cu   += _Cu;
	Ctdc += _Ctdc;
      }
    }
  }
  
}
