///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#if defined(WIN32) || defined(__WIN32__) // Windows specific so far

#ifndef _WISH_FREEIMAGE_READER_HH_
#define _WISH_FREEIMAGE_READER_HH_

#include "wishImageFileReader.hh"

#include "FreeImage.h"

namespace wish {
  //! FreeImageReader (OBS! Windows specific so far)
  //! For a FreeImageReader, the filename can be a pattern that matches
  //! a list of files that are read and stored in a single volume
  class FreeImageReader : public ImageFileReader {
    
  protected:
    /** Generic image loader
	@param lpszPathName Pointer to the full file name
	@param flag Optional load flag constant
	@return Returns the loaded dib if successful, returns NULL otherwise
    */
    static FIBITMAP* GenericLoader(const char* lpszPathName, int flag);

    //! path and pattern
    string m_path;
    string m_pattern;
    
    //! list of filenames
    vector<string> m_filelist; 
    
    //! no of files
    int nFiles() const {
      return int( m_filelist.size() );
    }
    
    //! help function for reading a scalar image sequence
    template<class T>
    bool readScalarSequence(Image<T> *im);
    
    //! help function for reading a color image sequence
    template<class T>
    bool readColorSequence(Image<T> *im);
    
    //! reads the header and stores the properties in the info struct
    bool readHeader();
    
    //! reads the data
    bool readData();
    
  public:
    //! constructor
    FreeImageReader() {}
    
    //! print self
    void printSelf() const {
      cout << "FreeImageReader";
    }
    
  };
  
}

#endif

#endif // Windows specific
