///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GL_CAMERA_HH_
#define _WISH_GL_CAMERA_HH_

#include "wishLinalg.hh"
#include <GL/glu.h>

namespace wish {

  // camera class
  class Camera {
  public:	
    Camera(const Vec3f &_eye=Vec3f(0,0,100),
	   const Vec3f &_at=Vec3f(0,0,0),
	   const Vec3f &_up=Vec3f(0,1,0)) :
      eye(_eye), at(_at), up(_up) {}
  
    virtual ~Camera() {}
  
    virtual void setEye(const Vec3f &_eye) { eye = _eye; }
    virtual void setAt(const Vec3f &_at) { at = _at; }
    virtual void setUp(const Vec3f &_up) { up = _up; }
  
    virtual Vec3f getEye() const { return eye; }
    virtual Vec3f getAt() const { return at; }
    virtual Vec3f getUp() const { return up; }
  
    // a view transformation
    virtual void viewTransformation() {
      gluLookAt(eye.x, eye.y, eye.z,
		at.x, at.y, at.z,
		up.x, up.y, up.z);
    }
  
  protected:	
    Vec3f eye, at, up;
  };

}
#endif
