///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GL_MATERIAL_HH_
#define _WISH_GL_MATERIAL_HH_

#include "wishColor.hh"

#include <GL/gl.h>

namespace wish {

  // material class
  class Material {
  public:
    Color4f ambient;
    Color4f diffuse;
    Color4f specular;
    Color4f emission;
    int shininess;
  
    // default = black
    Material(const Color4f &_ambient=Color4f(0.0f,0.0f,0.0f,1.0f),
	     const Color4f &_diffuse=Color4f(1.0f,1.0f,1.0f,1.0f),
	     const Color4f &_specular=Color4f(1.0f,1.0f,1.0f,1.0f),
	     const Color4f &_emission=Color4f(0.0f,0.0f,0.0f,1.0f),
	     int _shininess=10) :
      ambient(_ambient),
      diffuse(_diffuse),
      specular(_specular),
      emission(_emission),
      shininess(_shininess) {}
    
    Material(const Material &m) :
      ambient(m.ambient),
      diffuse(m.diffuse),
      specular(m.specular),
      emission(m.emission),
      shininess(m.shininess) {}
  
    void setAmbient(const Color4f &_ambient) {
      ambient = _ambient;      
    }
    void setDiffuse(const Color4f &_diffuse) {
      diffuse = _diffuse;    
    }
    void setSpecular(const Color4f &_specular) {
      specular = _specular;
    }
    void setEmission(const Color4f &_emission) {
      emission = _emission;
    }
    void setShininess(int _shininess) {
      shininess = _shininess;
    }
    void makeGL() {
      // set material properties
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient.getPointer());
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse.getPointer());
      glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular.getPointer());
      glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission.getPointer());
      glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
    }
    
  };

}
#endif
