///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GL_TEXT_HH_
#define _WISH_GL_TEXT_HH_

#include <GL/glut.h>
#include <string>
#include <iostream>

using namespace std;

namespace wish {
  
  //! Simple class for drawing bitmap fonts
  //! Code fetched from 
  class Text {
    
  protected:
    
    //! the string
    string str;
    
    //! position
    float xpos, ypos;
    
    //! width and height of the window
    int w, h;

    void setOrthographicProjection(int w, int h) {
      // switch to projection mode
      glMatrixMode(GL_PROJECTION);
      // save previous matrix which contains the 
      //settings for the perspective projection
      glPushMatrix();
      // reset matrix
      glLoadIdentity();
      // set a 2D orthographic projection
      gluOrtho2D(0, w, 0, h);
      // invert the y axis, down is positive
      glScalef(1, -1, 1);
      // move the origin from the bottom left corner
      // to the upper left corner
      glTranslatef(0.0f, float(-h), 0.0f);
      glMatrixMode(GL_MODELVIEW);
    }
    
    void resetPerspectiveProjection() {
      glMatrixMode(GL_PROJECTION);
      glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
    }
    
    void renderSpacedBitmapString(float x, 
				  float y,
				  const char *s,
				  int spacing=8, 
				  //void *font=GLUT_BITMAP_HELVETICA_18) {
				  //void *font=GLUT_BITMAP_8_BY_13) {
				  void *font=GLUT_BITMAP_9_BY_15) {
      const char *c;
      int x1=int(x);
      for (c=s; *c != '\0'; c++) {
	glRasterPos2f(float(x1), float(y));
    	glutBitmapCharacter(font, *c);
	x1 = x1 + glutBitmapWidth(font,*c) + spacing;
      }
    }
    
  public:
    //! constructor
    Text(string _str="", float _xpos=0.1, float _ypos=0.1) :
      str(_str), xpos(_xpos), ypos(_ypos)
    {}
    
    void setString(const string &_str) {
      str = _str;
    }
    
    //! set position (normalized viewport)
    void setPosition(float _xpos, float _ypos) {
      xpos = _xpos;
      ypos = _ypos;
    }
    
    void render() {
      w = glutGet(GLUT_WINDOW_WIDTH);
      h = glutGet(GLUT_WINDOW_HEIGHT);
      
      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glDisable(GL_TEXTURE_1D);
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_TEXTURE_3D);
      glDisable(GL_LIGHTING);
      glDisable(GL_DEPTH_TEST);

      glColor3f(1.0f,1.0f,1.0f);
      
      setOrthographicProjection(w,h);
      glPushMatrix();
      glLoadIdentity();
      
      renderSpacedBitmapString(xpos*w, ypos*h, str.c_str());
      
      glPopMatrix();
      resetPerspectiveProjection();
    
      glPopAttrib();
    }
    
  };
  
} // end namespace wish

#endif
