///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GPU_VOLUME_RENDERER_HH_
#define _WISH_GPU_VOLUME_RENDERER_HH_

#if defined (__WIN32__) || defined (WIN32)
#include <windows.h>
#endif

#include <GL/glew.h> // should always be included first!!!
#include <GL/glut.h>

#include "wishImage.hh"
#include "wishGLTexture.hh"

using namespace wish;
using namespace std;

namespace wish {

  // A GPU volume renderer 
  class GPUVolumeRenderer {

  protected:
    
    // textures
    vector<Texture*> textures;
    
    // shader stuff
    GLuint raycast_program; // raycaster
    
    GLint volume_handle;
    GLint volume_handle2;
    GLint blend_handle;
    GLint ray_step_handle;
    GLint wl_width_handle;
    GLint wl_level_handle;
    GLint total_extent_handle;
    
    // ray step
    float ray_step;
  
    // blending
    float blend;
    
    // window dimensions
    int window_width;
    int window_height;
  
    // window/level
    float wl_width;
    float wl_level;
    
    // shader ok
    bool shader_ok;

    // draw the volume cube
    void drawCube(float xp, float yp, float zp) const;
    
  public:
    
    // check frame buffer status
    static void checkFrameBufferStatus();
    
    // constructor
    GPUVolumeRenderer() : 
      raycast_program(0),
      volume_handle(0),
      volume_handle2(0),
      blend_handle(0),
      ray_step_handle(0),
      total_extent_handle(0),
      wl_width_handle(0),
      wl_level_handle(0),
      ray_step(.01f),
      blend(0.5),
      window_width(800),
      window_height(600),
      wl_width(1.0f),
      wl_level(0.5f),
      shader_ok( false )
    {}
  
    // OpenGL rendering (from display)
    void render() const;
  
    // update dimensions
    void reshape(int w, int h);
    
    // load shaders and build them, set up uniforms and fbo
    // shader_ok == false => no shaders are used
    void initialize( bool _shader_ok );
  
    // set/get window/level parameters
    void setWL(float _w, float _l) {
      wl_width = _w; wl_level = _l;
    }
    void setWLWidth(float _w) {
      wl_width = _w;
    }
    void setWLLevel(float _l) {
      wl_level = _l;
    }
    float getWLWidth() const {
      return wl_width;
    }
    float getWLLevel() const {
      return wl_level;
    }
  
    // set/get ray step
    void setRayStep(float _rs) {
      ray_step = _rs;
    }
    float getRayStep() const {
      return ray_step;
    }
    
    // set/get blend
    void setBlend(float _blend) {
      blend = _blend;
    }
    float getBlend() const {
      return blend;
    }
    
    // add texture
    void addTexture(Texture *tex) {
      textures.push_back(tex);
    }
    
  };

}

#endif
