///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm, Per Sundqvist
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishGVFConjugateGradients.hh"

namespace wishGVF {

  int pcg(float *u, float *rhs,float *vc,
	  float *r, float *p, float *w, float *v,
	  float MU, float AR2,int X, int Y, int Z,
	  float eps,float *a,float omega){
    float alpha, beta, beta_num, beta_denom, temp; //, mumu;
    float resnorm = 1.0f;
    int iter = 1;
    int  k, XY; //, x, y, z;
    XY = X*Y;

    // Compute residual of initial guess: rhs-Au
    for(int z=1; z<Z-1; z++) {
      for(int y=1; y<Y-1; y++) {
	for(int x=1; x<X-1; x++) {
	  k = z*XY+y*X+x;
	  r[k] = rhs[k]-appl_op_loc(u,vc,MU,AR2,k,X,XY);	
	}
      }
    }
    beta_num = 0;

    // First iteration
    // Solve Mv=r
    appl_precond(v,r,vc,a,MU,AR2,X,Y,Z,omega);
    for(int z=Z-2; z>0; z--) {
      for(int y=Y-2; y>0; y--) {
	for(int x=X-2; x>0; x--) {
	  k = z*XY+y*X+x;
	  p[k] = v[k];
	  beta_num = beta_num + r[k]*v[k]; 
	}
      }
    }
    // w = Ap and alpha 
    temp = 0.0f;
    for(int z=1; z<Z-1; z++) {
      for(int y=1; y<Y-1; y++) {
	for(int x=1; x<X-1; x++) {
	  k = z*XY+y*X+x;
	  w[k] = appl_op_loc(p,vc,MU,AR2,k,X,XY);	
	  temp = temp+p[k]*w[k];
	}
      }
    }
    alpha = beta_num/temp;

    // u = u+alpha*p,  r = r-alpha*w,  and rho = (r,r);
    for(int z=1; z<Z-1; z++) {
      for(int y=1; y<Y-1; y++) {
	for(int x=1; x<X-1; x++) {
	  k = z*XY+y*X+x;
	  u[k] = u[k]+alpha*p[k];
	  r[k] = r[k]-alpha*w[k];
	}
      }
    }
    resnorm = residual_maxnorm(rhs,u,vc,MU,AR2,X,Y,Z);

    // Iterate until solution is as good as before
    while( resnorm > eps*4 ) {
      // Solve Mv=r
      beta_denom = beta_num;
      beta_num = 0;
      iter++;
      appl_precond(v,r,vc,a,MU,AR2,X,Y,Z,omega);
   
      for(int z=Z-2; z>0; z--) {
	for(int y=Y-2; y>0; y--) {
	  for(int x=X-2; x>0; x--) {
	    k = z*XY+y*X+x;
	    beta_num = beta_num + r[k]*v[k]; 
	  }
	}
      }
      beta  = beta_num/beta_denom;

      // p = v+beta*p
      for(int z=1; z<Z-1; z++) {
	for(int y=1; y<Y-1; y++) {
	  for(int x=1; x<X-1; x++) {
	    k = z*XY+y*X+x;
	    p[k] = v[k]+beta*p[k];
	  }
	}
      }
     
      // w = Ap and alpha 
      temp = 0.0f;
      for(int z=1; z<Z-1; z++) {
	for(int y=1; y<Y-1; y++) {
	  for(int x=1; x<X-1; x++) {
	    k = z*XY+y*X+x;
	    w[k] = appl_op_loc(p,vc,MU,AR2,k,X,XY);	
	    temp = temp+p[k]*w[k];
	  }
	}
      }
      alpha = beta_num/temp;

      // u = u+alpha*p,  r = r-alpha*w,  and rho = (r,r);
      for(int z=1; z<Z-1; z++) {
	for(int y=1; y<Y-1; y++) {
	  for(int x=1; x<X-1; x++) {
	    k = z*XY+y*X+x;
	    u[k] = u[k]+alpha*p[k];
	    r[k] = r[k]-alpha*w[k];
	  }
	}
      }
   
      resnorm = residual_maxnorm(rhs,u,vc,MU,AR2,X,Y,Z);
      std::cout << "max residual = " << resnorm << std::endl;    
    }  //while-end
    return iter;
  }

  //
  void appl_precond(float *v, float *r, float *vc, float *a, 
		    float MU, float AR2, int X, int Y, int Z, float omega){

    // This is where different preconditioners can be chosen for PCG 
    // The definitions of the preconditioners are found in pSolverUtil.hh

    //  for(int i=0;i<X*Y*Z;i++) v[i]=r[i]; // No precondiitoner, Iv=r
    // IC(v, r, a, MU, AR2, X, Y, Z);       // Incomplete Cholesky
    MIC(v, r, a, MU, AR2, X, Y, Z);       // Modified IC
    //  SSOR_precond(v, r, vc, MU, AR2, X, Y, Z, omega); 
    //  jacobi_precond(v, r, vc, X, Y, Z, omega);

  }

}
