///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm, Per Sundqvist
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GVF_MULTIGRID_HH_
#define _WISH_GVF_MULTIGRID_HH_

#include "wishGVFSolverUtil.hh"

namespace wishGVF {
  //
  // Multigrid method, here with three grids
  // First a full multigrid V-cycle and then V-cycles until convergence
  int multigrid(float *u, float *u2, float *u3,       // gridfunctions
		float *rhs, float *rhs2, float *rhs3, // right hand sides
		float *vc, float *vc2, float *vc3,    // variable diagonals
		float MU, float AR,                   // coefficients to operator
		int Xp, int Yp, int Zp,               // largest sizes
		float eps,                            // termination criterion
		int nu1, int nu2, int nu3,            // parameters for MG
		float omega);                         // parameter for smoother

  //
  // Prolongation of standard type, i.e. trilinear interpolation 
  void prolongate(float *fine,          // fine grid vector, the output
		  float *coarse,        // coarse grid vector, the input
		  int X, int Y, int Z); // sizes of fine grid vector

  //
  // Restriction of "full weighting" type, very standard
  // Would possibly be faster if done by accumulation in temp-vector
  void restrict(float *coarse,       // coarse grid vector, the output
		float *fine,         // fine grid vector, the input
		int X, int Y, int Z);// sizes of fine grid vector

  //
  // Smoother, also called relaxation, here Jacobi, SOR, or SSOR
  void pre_smoother(float *u,             // gridfunction
		    float *rhs,           // right hand side
		    float *vc,            // variable coefficient
		    float MU, float AR,   // coefficients for operator
		    int X, int Y, int Z,  // sizes
		    int nu,               // number of smoothing steps
		    float omega,         // relaxation parameter
		    float *temp);
  //
  // Smoother, also called relaxation, here Jacobi, SOR, or SSOR
  void post_smoother(float *u,             // gridfunction
		     float *rhs,           // right hand side
		     float *vc,            // variable coefficient
		     float MU, float AR,   // coefficients for operator
		     int X, int Y, int Z,  // sizes
		     int nu,               // number of smoothing steps
		     float omega,         // relaxation parameter
		     float *temp);

  //
  // "Solver" for the problem at the coarsest grid
  void coarse_solve(float *u,             // gridfunction
		    float *rhs,           // right hand side
		    float *vc,            // variable coefficient
		    float MU, float AR,   // coefficients for operator
		    int X, int Y, int Z,  // sizes
		    int nu,               // number of smoothing steps
		    float omega,         // relaxation parameter
		    float *temp);

  //
  // Constructs the variable coefficients on the coarser grids
  void mg_setup(float *vc,              // variable coefficient on fines grid
		float *vc2, float *vc3, // ... on coarser grids
		float MU, float AR2,     // coefficients for operator
		int Xp, int Yp, int Zp);// sizes of fine grid

}
#endif
