///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GAUSSIAN_GRADIENT_MAGNITUDE_FILTER_HH_
#define _WISH_GAUSSIAN_GRADIENT_MAGNITUDE_FILTER_HH_

#include "wishFilterObject.hh"
#include "wishGaussianGradientFilter.hh"

namespace wish {
  
  //! GaussianGradientMagnitudeFilter
  
  template<class InputImage, class OutputImage>
  class GaussianGradientMagnitudeFilter : 
    public FilterObject<InputImage, OutputImage> {
    
  public:
    
    //! pointer typedef
    typedef auto_ptr< GaussianGradientMagnitudeFilter<InputImage,OutputImage> > 
    Pointer;
    typedef typename InputImage::Pointer InputImagePointer;
    typedef typename OutputImage::Pointer OutputImagePointer;
    typedef typename InputImage::VoxelType InputImageVoxelType;
    typedef typename OutputImage::VoxelType OutputImageVoxelType;
    
    //! constructor
    GaussianGradientMagnitudeFilter() : 
      m_sigma(1.0),
      m_max_kernel_radius(20)
    {}
    
    //! set sigmas
    void setSigma(double sigma) {
      m_sigma = sigma;
    }
    
    //! set radius
    void setMaxKernelRadius(int radius) {
      m_max_kernel_radius = radius;
    }
    
  protected:
    
    double m_sigma;
    int m_max_kernel_radius;
    
    virtual void process();
    
  private:
    // purposely not implemented
    GaussianGradientMagnitudeFilter(const GaussianGradientMagnitudeFilter&);
    GaussianGradientMagnitudeFilter& operator=(const GaussianGradientMagnitudeFilter&);
  };
  
  template<class InputImage, class OutputImage>
  void GaussianGradientMagnitudeFilter<InputImage, OutputImage>::process() {
    
    InputImage &in = *(this->m_input);
    OutputImage &out = *(this->m_output);
    
    GaussianGradientFilter<InputImage> gradfilt;
    gradfilt.setSigma(m_sigma);
    gradfilt.setMaxKernelRadius(m_max_kernel_radius);
    gradfilt.setInput(&in);
    gradfilt.execute();
    
    Image<Vec3f> &vecim = *(gradfilt.getOutput());
    
    out.resize( in );
    out.setLabel( in.getLabel() + "_GGM" );
    
    for(int i=0; i<out.totalSize(); i++) 
      out[i] = OutputImageVoxelType( vecim[i].norm() );
  }
  
} // end namespace wish

#endif
