///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_GRADIENT_MAGNITUDE_FILTER_HH_
#define _WISH_GRADIENT_MAGNITUDE_FILTER_HH_

#include "wishFilterObject.hh"

namespace wish {
  
  //! GradientMagnitudeFilter
  
  template<class InputImage, class OutputImage>
  class GradientMagnitudeFilter : 
    public FilterObject<InputImage, OutputImage> {
    
  public:
    
    //! pointer typedef
    typedef auto_ptr< GradientMagnitudeFilter<InputImage,OutputImage> > 
    Pointer;
    typedef typename InputImage::Pointer InputImagePointer;
    typedef typename OutputImage::Pointer OutputImagePointer;
    typedef typename InputImage::VoxelType InputImageVoxelType;
    typedef typename OutputImage::VoxelType OutputImageVoxelType;
    
    //! constructor
    GradientMagnitudeFilter() {}
    
  protected:
    virtual void process();
    
  private:
    // purposely not implemented
    GradientMagnitudeFilter(const GradientMagnitudeFilter&);
    GradientMagnitudeFilter& operator=(const GradientMagnitudeFilter&);
  };
  
  template<class InputImage, class OutputImage>
  void GradientMagnitudeFilter<InputImage, OutputImage>::process() {
    
    this->m_output->resize( *(this->m_input) );
    this->m_output->reset( OutputImageVoxelType(0) );
    
    InputImage &in   = (*(this->m_input));
    OutputImage &out = (*(this->m_output));
    
    for(int z=0; z<in.depth(); z++) {
      for(int y=0; y<in.height(); y++) {
	for(int x=0; x<in.width(); x++) {
	  Vec3f grad = in.getGradient(x,y,z);
	  out(x,y,z) = OutputImageVoxelType( grad.norm() );
	}
      }
    }
    
  }
  
} // end namespace wish

#endif
