///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishImageFileReader.hh"
#include "wishVTKFileReader.hh"
#include "wishPixyFileReader.hh"

#if defined(WIN32) || defined (__WIN32__)
#include "wishFreeImageReader.hh"
#endif

namespace wish {
  
  // factory
  ImageFileReader* ImageFileReader::Create(const string &filename) {
  
    ImageFileReader *ifr=0;
    if( filename.find(".vtk") != string::npos ) {
      ifr = new VTKFileReader;
      ifr->setFilename(filename);

      cerr << "VTKFileReader used" << endl;
    }
    else if( filename.find(".ipad") != string::npos ||
	     filename.find(".pixy") != string::npos ) {
      ifr = new PixyFileReader;
      ifr->setFilename(filename);

      cerr << "PixyFileReader used" << endl;
    }

#if defined(WIN32) || defined (__WIN32__)
    else if( filename.find(".jpg")  != string::npos ||
	     filename.find(".jpeg") != string::npos ||
	     filename.find(".png")  != string::npos ||
	     filename.find(".tif")  != string::npos ||
	     filename.find(".tiff") != string::npos ||
	     filename.find(".bmp")  != string::npos ||
	     filename.find(".gif")  != string::npos ) {
      ifr = new FreeImageReader;
      ifr->setFilename(filename);

      cerr << "FreeImageReader used" << endl;
    }
#endif

    return ifr;
  }
  
}
