///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishIndexedTriangleSet.hh"

namespace wish {
  
  IndexedTriangleSet::IndexedTriangleSet() { 
  }
  
  void IndexedTriangleSet::clear() {
    indexList.clear();
    vertexList.clear();
    normalList.clear();
  }
  
  void IndexedTriangleSet::normalizeNormals() {
    vector<Vec3f>::iterator nl;
    nl = normalList.begin();
    while(nl != normalList.end()) {
      float no = sqrt( (*nl).x * (*nl).x + 
		       (*nl).y * (*nl).y +
		       (*nl).z * (*nl).z ); 
      
      
      if(no > constants::eps9) {
	(*nl).x /= no;
	(*nl).y /= no;
	(*nl).z /= no;
      }
      nl++;  
    }
  }
  
  void IndexedTriangleSet::drawGL(int mode, Colormap *cmap) {

    glPushAttrib( GL_ALL_ATTRIB_BITS );
    
    if( cmap != 0 ) {
      glEnable(GL_COLOR_MATERIAL);
      glColor3fv(cmap->getColor(0.0));
    }
    
    if( mode == 0 ) {
      // GLwireframe rendering
      glLineWidth(1.0);
      glEnable(GL_LINE_SMOOTH);
      glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
      
      glBegin(GL_LINES);	
      for(int i=0; i<this->nrFaces(); i++) {
	glNormal3fv(normalList[indexList[3*i]]);
	glVertex3fv(vertexList[indexList[3*i]]);
	
	glNormal3fv(normalList[indexList[3*i+1]]);
	glVertex3fv(vertexList[indexList[3*i+1]]);
	
	glNormal3fv(normalList[indexList[3*i+1]]);
	glVertex3fv(vertexList[indexList[3*i+1]]);
	
	glNormal3fv(normalList[indexList[3*i+2]]);
	glVertex3fv(vertexList[indexList[3*i+2]]);
	
	glNormal3fv(normalList[indexList[3*i+2]]);
	glVertex3fv(vertexList[indexList[3*i+2]]);
	
	glNormal3fv(normalList[indexList[3*i]]);
	glVertex3fv(vertexList[indexList[3*i]]);
      }
      glEnd();
    }
    
    else if( mode == 1 ) {
      
      // GLflat rendering
      glBegin(GL_TRIANGLES);
      for(int i=0; i<this->nrFaces(); i++) {
	glNormal3fv(normalList[indexList[3*i]]);
	glVertex3fv(vertexList[indexList[3*i]]);
	glVertex3fv(vertexList[indexList[3*i+1]]);
	glVertex3fv(vertexList[indexList[3*i+2]]);
      }
      glEnd();
      
    }
    
    else if( mode == 2 ) {
      
      // GLsmooth rendering
      glBegin(GL_TRIANGLES);
      for(int i=0; i<this->nrFaces(); i++) {
	glNormal3fv(normalList[indexList[3*i]]);
	glVertex3fv(vertexList[indexList[3*i]]);
	
	glNormal3fv(normalList[indexList[3*i+1]]);
	glVertex3fv(vertexList[indexList[3*i+1]]);
	
	glNormal3fv(normalList[indexList[3*i+2]]);
	glVertex3fv(vertexList[indexList[3*i+2]]);
      }
      glEnd();
      
    }   
    
    glPopAttrib();        
    
  }
  
}
