///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_INDEXED_TRIANGLE_SET_HH_
#define _WISH_INDEXED_TRIANGLE_SET_HH_

#if defined (__WIN32__) || defined (WIN32)
#include <windows.h>
#endif
#include <GL/gl.h>

#include "wish.hh"
#include "wishLinalg.hh"
#include "wishColormap.hh"

namespace wish {

  class IsoSurfaceRenderer;
  
  class IndexedTriangleSet {
  public:
    
    friend class IsoSurfaceRenderer;
    
    IndexedTriangleSet();
    
    // clear lists
    void clear();
    
    // normal calculations
    void computeNormals() {}
    void normalizeNormals();
    
    // OpenGL drawing
    void drawGL(int mode=0, Colormap *cmap=0);
    
    // get/set
    const vector<Vec3f>& getVertexList() const {
      return vertexList;
    }
    const vector<Vec3f>& getNormalList() const {
      return normalList;
    }
    const vector<unsigned int>& getIndexList() const {
      return indexList;
    }
    
    // sizes
    int nrVertices() const {
      return (int) vertexList.size();
    }
    
    int nrFaces() const {
      return (int) indexList.size()/3;
    }
    
  protected:
    
    // the data
    vector<Vec3f> vertexList;
    vector<Vec3f> normalList;
    vector<unsigned int> indexList;
    
  };
  
}
#endif

