///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_MAX_HEAP_HH_
#define _WISH_MAX_HEAP_HH_

#include "wish.hh"
#include "wishMinHeap.hh"

namespace wish {
  
  //! the maximum heap class
  //!
  //! Implements a maximum heap data structure where each element in the heap
  //! also contains an index that tells where in the heap it is located. This
  //! allows for faster "upHeap" operations. The max heap is used in 
  //! wishFuzzyConnectednessFilter. For more general priority queues, 
  //! use stl:s priority_queue or heap. 
  template<class T>
  class MaxHeap : public MinHeap<T> {
  public:
    
    MaxHeap(int _capacity=100000) : MinHeap<T>(_capacity) {}
    
    virtual bool compare(const int &a, const int &b) {
      return this->getElement(a)->value > this->getElement(b)->value;
    }
    
    virtual bool compare2(const int &a, const int &b) {
      return this->getElement(a)->value >= this->getElement(b)->value;
    }
    
  private:
    // purposely not implemented
    MaxHeap(const MaxHeap &mh);
    MaxHeap& operator=(const MaxHeap &mh);
  };
  
} // end namespace wish

#endif

  
