///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_POTENTIAL_DISTANCE_MAP_FILTER_HH_
#define _WISH_POTENTIAL_DISTANCE_MAP_FILTER_HH_

#include "wishFilterObject.hh"
#include "wishDistanceMapFilter.hh"

namespace wish {
  

  // The potential distance map filter computes the distance from a
  // binary object. The distance is zero at the border and increases 
  // away from the border.
  template<class InputImage, class OutputImage>
  class PotentialDistanceMapFilter : 
    public FilterObject<InputImage, OutputImage> {
    
  public:
    
    //! pointer typedef
    typedef auto_ptr< PotentialDistanceMapFilter<InputImage,OutputImage> > 
    Pointer;
    typedef typename InputImage::Pointer InputImagePointer;
    typedef typename OutputImage::Pointer OutputImagePointer;
    typedef typename InputImage::VoxelType InputImageVoxelType;
    typedef typename OutputImage::VoxelType OutputImageVoxelType;
    
    //! constructor
    PotentialDistanceMapFilter() : 
      m_weight_a(3.0), 
      m_weight_b(4.0), 
      m_weight_c(5.0)
    {}
    
    //! set weights
    void setWeights(double _weight_a=3.0, 
		    double _weight_b=4.0, 
		    double _weight_c=5.0) {
      m_weight_a = _weight_a;
      m_weight_b = _weight_b;
      m_weight_c = _weight_c;
    }
    
  protected:
    double m_weight_a, m_weight_b, m_weight_c;
    
    virtual void process();
    
  private:
    // purposely not implemented
    PotentialDistanceMapFilter(const PotentialDistanceMapFilter&);
    PotentialDistanceMapFilter& operator=(const PotentialDistanceMapFilter&);
  };
  
  template<class InputImage, class OutputImage>
  void PotentialDistanceMapFilter<InputImage, OutputImage>::process() {
    
    this->m_output->resize( *(this->m_input) );
    this->m_output->reset( OutputImageVoxelType(0) );
    
    InputImage &in = *(this->m_input);
    OutputImage &out = *(this->m_output);
    OutputImage temp;
    
    DistanceMapFilter<InputImage,OutputImage> dist;
    dist.setWeights(m_weight_a, m_weight_b, m_weight_c);
    dist.setInput(&in);
    dist.setOutput(&temp);
    dist.setInternal(true);
    dist.execute();
    
    dist.setOutput(0);
    dist.setInternal(false);
    dist.execute();
    
    out = *(dist.getOutput())+temp;
    out.setLabel( in.getLabel() + "_PDMF" );
  }    
  
} // end namespace wish

#endif
