///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_RAW_FILE_READER_HH_
#define _WISH_RAW_FILE_READER_HH_

#include "wishImageFileReader.hh"

namespace wish {
  
  //! RawFileReader
  class RawFileReader : public ImageFileReader {
    
  protected:
    //! little or big endian
    bool m_little_endian;
    
    //! reads the header and stores the properties in the info struct
    bool readHeader();
    
    //! reads the data
    bool readData();
    
  public:
    //! constructor
    RawFileReader() : m_little_endian(true) {}
    
    //! print self
    void printSelf() const {
      cout << "RawFileReader";
    }
    
    void setDimensions( const Vec3i &v ) {
      this->m_info.dim = v;
    }
    
    void setSpacing( const Vec3f &v ) {
      this->m_info.spc = v;
    }
    
    void setDatatype( const string &str ) {
      this->m_info.datatype = str;
    }
    
    void setHeaderSize( int hs ) {
      this->m_info.headersize = hs;      
    }
    
    void setLittleEndian( bool le ) {
      m_little_endian = le;
    }
    
  };
  
}

#endif
