///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_RENUMBER_FILTER_HH_
#define _WISH_RENUMBER_FILTER_HH_

#include "wishFilterObject.hh"
#include "wishNeighborhoodIterator.hh"

namespace wish {
  
  //! renumbers labeled regions so that they are numbered from 1 to ...
  template<class InputImage, class OutputImage>
  class RenumberFilter : public FilterObject<InputImage, OutputImage> {
    
  public:
    
    //! pointer typedef
    typedef auto_ptr< RenumberFilter<InputImage,OutputImage> > Pointer;
    typedef typename InputImage::Pointer InputImagePointer;
    typedef typename OutputImage::Pointer OutputImagePointer;
    typedef typename InputImage::VoxelType InputImageVoxelType;
    typedef typename OutputImage::VoxelType OutputImageVoxelType;
    
    //! constructor
    RenumberFilter() {}
    
  protected:
    virtual void process();
    
  private:
    
    RenumberFilter(const RenumberFilter&);
    RenumberFilter& operator=(const RenumberFilter&);
  };
  
  template<class InputImage, class OutputImage>
  void RenumberFilter<InputImage, OutputImage>::process() {
    
    this -> m_output -> resize( *(this->m_input) );
    this -> m_output -> reset( OutputImageVoxelType(0) );
    
    InputImage &in = this->m_input;
    OutputImage &out = this->m_output;
    
    long current=0;
    vector<InputImageVoxelType> labels(in.maxValue()+1,0);  
    for(long i=0; i<in.totalSize(); i++) {
      if( in[i] > 0 && labels[in[i]] == 0 ) {
	labels[in[i]] = ++current;
	out[i] = current;
      }
      else {
	out[i] = labels[in[i]];
      }
    }
    
  }
  
} // end namespace wish

#endif
