///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_SHADER_UTILITIES_HH_
#define _WISH_SHADER_UTILITIES_HH_

#if defined (__WIN32__) || defined (WIN32)
#include <windows.h>
#endif

#include <GL/glew.h> // should always be included first!!!
#include <GL/glut.h>

#include "wish.hh"

namespace wish {
  
// macro for checking GL extensions through GLEW
#define checkExtension(_ext) _checkExtension( GLEW_##_ext, #_ext )
  
  inline bool _checkExtension( size_t ext, const string &exts ) {
    bool ok = (ext!=0);
    if( !ok ) cerr << "Extension " << exts << " not found!" << endl;
    else cerr << "Extension " << exts << " OK!" << endl;
    return ok;
  }
  
  // load a shader program from file
  // parameter type is GL_VERTEX_SHADER_ARB or GL_FRAGMENT_SHADER_ARB
  GLhandleARB buildShaderFromFile(const string &filename, 
				  GLenum type);
  
  // create shader program
  GLhandleARB createShaderProgram(const string &vs, 
				  const string &fs);
  
}

#endif
