///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishutil.hh"

namespace wish {

  namespace util {
    
    // split filename (not very nice...)
    void splitFilename(const string &fn, 
		       string &dir,
		       string &name,
		       string &ext) {
      int slashpos = (int) fn.rfind('\\');
      int slashpos2 = (int) fn.rfind('/');
      
      if( (string::size_type) slashpos != string::npos &&
	  (string::size_type) slashpos2 != string::npos )
	slashpos = maxOf(slashpos, slashpos2);
      else if( (string::size_type) slashpos2 != string::npos )
	slashpos = slashpos2;
      else
	slashpos = -1;
      
      int dotpos = (int) fn.rfind('.');
      
      dir = fn.substr(0,slashpos+1);
      name = fn.substr(slashpos+1,dotpos-(slashpos+1));
      ext = fn.substr(dotpos);
    }
      
    // system command
    bool systemCommand(const std::string &cmd) {
      int sysok;
      sysok = std::system(cmd.c_str());
      return (sysok != -1);
    }
    
    // check filesize
    int fileSize(const std::string &filename) {
      std::ifstream is(filename.c_str(), ios::binary);
      int length;
      is.seekg(0, std::ifstream::end);
      length = is.tellg();
      is.close();
      return length;
    }
    
  } // namespace util
  
} // namespace wish

