#
#
# Copyright 2007, Erik Vidholm
#
#
#   This file is part of WISH.
#
#   WISH is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   WISH is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with WISH.  If not, see <http:#www.gnu.org/licenses/>.
#
#

from H3D import *
from H3DInterface import *

import os

# The python script with Tkinter GUI
import PSTK

# The main group
maingroup, = references.getValue()

# Import scene from file and add it to the children field 
group, dn = createX3DFromURL( "demo_scene.x3d" )
maingroup.children.push_back( group )

# the device
di = getActiveDeviceInfo()
phantom = di.device.getValue()[0]

# grasp transform
phantom.mainButton.route( dn["GRASP_TRANSFORM"].grasp )
phantom.trackerPosition.route( dn["GRASP_TRANSFORM"].graspPosition )
phantom.trackerOrientation.route( dn["GRASP_TRANSFORM"].graspOrientation )

# mouse rotation
class MouseRotation(TypedField(SFRotation, (SFBool, SFVec2f))):
  def __init__( self ):
    SFRotation.__init__( self )
    
  def update( self, event ):
    pressed = self.getRoutesIn()[0].getValue()
    motion = self.getRoutesIn()[1].getValue()

    if( pressed and motion*motion > 1e-9 ):
      # the left mouse button is pressed and the mouse has moved so 
      # we add a rotation based on the motion.
      perp = Vec2f( motion.y, motion.x )
      perp.normalize()
      return Rotation( perp.x, perp.y, 0, motion.length() * 0.01 ) * \
             self.getValue()
    
    else:
      return self.getValue()

ms = createX3DNodeFromString("<MouseSensor />")[0]
mouseRotation = MouseRotation()
ms.leftButton.route( mouseRotation )
ms.motion.route( mouseRotation )
mouseRotation.route( dn["VOLUME_TRANSFORM"].rotation )

###
# set the stylus to be the VHTK stylus
mpr = os.environ.get("WISH_ROOT");
stylus = createX3DFromURL( mpr+"/wishH3D/demo/vhtk_model_stylus.x3d" )[0]
[st, dntmp] = createX3DFromString( '<Transform DEF="ST" />' )
dntmp["ST"].children.push_back( stylus )

# add the stylus to the scene (references[0])
phantom.stylus.setValue( createX3DNodeFromString("<Shape />")[0] )
group.children.push_back( dntmp["ST"] )

## default route
phantom.trackerPosition.route( dntmp["ST"].translation )
phantom.trackerOrientation.route( dntmp["ST"].rotation )

## if volume haptics is present 
dn["HAPTICS"].proxyWorldPosition.route( dntmp["ST"].translation )
###

# SpaceWareSensor
sws = createX3DNodeFromString("<SpaceWareSensor/>")[0]
sws.rotationScale.setValue(0.05)
maingroup.children.push_back( sws ) # must belong to scenegraph

# SpaceWare buttons
swb = createX3DNodeFromString("""
<PythonScript url="spacemousebuttons.py"/>""")[0]
maingroup.children.push_back( swb ) # must belong to scenegraph
sws.latestButtonPress.route( swb.buttonController )
sws.latestButtonRelease.route( swb.buttonController )

# --- Volume ---
#sws.accumulatedRotation.route( dn["VOLUME_TRANSFORM"].rotation )
dn["VOLUME"].smartScaling.route( dn["VOLUME_TRANSFORM"].scale )
dn["VOLUME"].currentImage.route( dn["SLICERTEXTURE"].imageIndex )

# --- GUI ---
dn["VOLUME"].imageLabels.route( PSTK.imageLabels )

# File routings
PSTK.fileFrame.fileOpen.route( dn["VOLUME"].loadImage )
PSTK.fileFrame.fileNew.route( dn["VOLUME"].newImage )
PSTK.fileFrame.fileSave.route( dn["VOLUME"].saveImage )
PSTK.fileFrame.fileClear.route( dn["VOLUME"].clearImages )
PSTK.fileFrame.fileQuit.route( dn["VOLUME"].quitAPI )

PSTK.fileFrame.width.route( dn["VOLUME"].uwidth )
PSTK.fileFrame.height.route( dn["VOLUME"].uheight )
PSTK.fileFrame.depth.route( dn["VOLUME"].udepth )

class FloatToVec3(TypedField(SFVec3f, (SFFloat, SFFloat, SFFloat))):
  def update( self, event ):
    x = self.getRoutesIn()[0].getValue()
    y = self.getRoutesIn()[1].getValue()
    z = self.getRoutesIn()[2].getValue()
    return Vec3f(x,y,z)

floatToVec3 = FloatToVec3()

PSTK.fileFrame.spacingX.route( floatToVec3 )
PSTK.fileFrame.spacingY.route( floatToVec3 )
PSTK.fileFrame.spacingZ.route( floatToVec3 )

floatToVec3.route( dn["VOLUME"].uspacing )


# Volume routings
PSTK.volumeFrame.desiredSize.route( dn["VOLUME"].desiredSize )
PSTK.volumeFrame.current.menuIndex.route( dn["VOLUME"].currentImage )
PSTK.volumeFrame.overlay.menuIndex.route( dn["OVERLAY"].imageIndex )
PSTK.volumeFrame.secondary.menuIndex.route( dn["SECONDARYTEXTURE"].imageIndex )
PSTK.volumeFrame.window.route( dn["SLICER"].window )
PSTK.volumeFrame.level.route( dn["SLICER"].level )
PSTK.volumeFrame.renderMode.route( dn["SLICER"].renderMode )
PSTK.volumeFrame.activeX.route( dn["SLICER"].activeX )
PSTK.volumeFrame.activeY.route( dn["SLICER"].activeY )
PSTK.volumeFrame.activeZ.route( dn["SLICER"].activeZ )
PSTK.volumeFrame.hapticButton.route( dn["SLICER"].hapticsEnabled )

# Haptics routings
class HapticModeSwitcher( AutoUpdate(SFInt32) ):
  def update(self, event):
    m = event.getValue()
    if m==1:
      dn["HAPTICS"].hapticMode.setValue( dn["HAPTICS_VISCOSITY_MODE"] )
      print "Viscosity mode"
    elif m==2:
      dn["HAPTICS"].hapticMode.setValue( dn["HAPTICS_POTENTIAL_MODE"] )
      print "Potential mode"
    elif m==3:
      dn["HAPTICS"].hapticMode.setValue( dn["HAPTICS_VECTOR_MODE"] )
      print "Vector mode"
    else: #elif m==0:
      dn["HAPTICS"].hapticMode.setValue( dn["HAPTICS_SURFACE_MODE"] )
      print "Surface mode"
      
    return m

hapticModeSwitcher = HapticModeSwitcher()
PSTK.hapticsFrame.hapticMode.route( hapticModeSwitcher )

PSTK.hapticsFrame.image.menuIndex.route( dn["HAPTICS"].imageIndex )
PSTK.hapticsFrame.activeButton.route( dn["HAPTICS"].active )
PSTK.hapticsFrame.onlyXY.route( dn["HAPTICS"].onlyXY )
PSTK.hapticsFrame.stiffness.route( dn["HAPTICS"].stiffness )

PSTK.hapticsFrame.viscosityAlpha.route( dn["VISCOSITYTF"].alpha )
PSTK.hapticsFrame.viscosityBeta.route( dn["VISCOSITYTF"].beta )
PSTK.hapticsFrame.strength.route( dn["STRENGTHTF"].maxvalue )
PSTK.hapticsFrame.strengthAlpha.route( dn["STRENGTHTF"].alpha )
PSTK.hapticsFrame.strengthBeta.route( dn["STRENGTHTF"].beta )
PSTK.hapticsFrame.friction.route( dn["FRICTIONTF"].value )
PSTK.hapticsFrame.distinctnessAlpha.route( dn["DISTINCTNESSTF"].alpha )
PSTK.hapticsFrame.distinctnessBeta.route( dn["DISTINCTNESSTF"].beta )
PSTK.hapticsFrame.flipDirection.route( dn["HAPTICS_SURFACE_MODE"].negativeGradientAsNormal )
PSTK.hapticsFrame.flipDirection.route( dn["HAPTICS_POTENTIAL_MODE"].negativeGradient )
PSTK.hapticsFrame.flipDirection.route( dn["HAPTICS_VECTOR_MODE"].negativeDirection )

# Edit volume
dn["VOLUME"].voxelPosition.route( dn["OVERLAY"].editPosition )
swb.button1.route( dn["OVERLAY"].editAdd )
swb.button2.route( dn["OVERLAY"].editRemove )
PSTK.editTextureFrame.editSize.route( dn["OVERLAY"].editSize )
PSTK.editTextureFrame.image.menuIndex.route( dn["OVERLAY"].imageIndex )
PSTK.editTextureFrame.editLabel.route( dn["OVERLAY"].editLabel )
PSTK.editTextureFrame.clearAll.route( dn["OVERLAY"].clearAll )
PSTK.editTextureFrame.clearLabel.route( dn["OVERLAY"].clearLabel )

# Slicer routings
phantom.secondaryButton.route( dn["SLICER"].grab )
dn["VOLUME"].proxyPosition.route( dn["SLICER"].grabPosition )

# Threshold routings
PSTK.thresholdFrame.input.menuIndex.route( dn["THRESHOLDFILTER"].inputIndex )
PSTK.thresholdFrame.output.menuIndex.route( dn["THRESHOLDFILTER"].outputIndex )
PSTK.thresholdFrame.lowThreshold.route( dn["THRESHOLDFILTER"].lowThreshold )
PSTK.thresholdFrame.highThreshold.route( dn["THRESHOLDFILTER"].highThreshold )
PSTK.thresholdFrame.execute.route( dn["THRESHOLDFILTER"].executeFilter )

# Gaussian smoothing routings
PSTK.gsFrame.input.menuIndex.route( dn["GSFILTER"].inputIndex )
PSTK.gsFrame.output.menuIndex.route( dn["GSFILTER"].outputIndex )
PSTK.gsFrame.sigma.route( dn["GSFILTER"].sigma )
PSTK.gsFrame.execute.route( dn["GSFILTER"].executeFilter )

# Gaussian gradient magnitude routings
PSTK.ggmFrame.input.menuIndex.route( dn["GGMFILTER"].inputIndex )
PSTK.ggmFrame.output.menuIndex.route( dn["GGMFILTER"].outputIndex )
PSTK.ggmFrame.sigma.route( dn["GGMFILTER"].sigma )
PSTK.ggmFrame.execute.route( dn["GGMFILTER"].executeFilter )

# Gaussian gradient routings
PSTK.ggFrame.input.menuIndex.route( dn["GGFILTER"].inputIndex )
PSTK.ggFrame.output.menuIndex.route( dn["GGFILTER"].outputIndex )
PSTK.ggFrame.sigma.route( dn["GGFILTER"].sigma )
PSTK.ggFrame.outputRGBA.route( dn["GGFILTER"].outputRGBA )
PSTK.ggFrame.execute.route( dn["GGFILTER"].executeFilter )

# GVF routings
PSTK.gvfFrame.input.menuIndex.route( dn["GVFFILTER"].inputIndex )
PSTK.gvfFrame.output.menuIndex.route( dn["GVFFILTER"].outputIndex )
PSTK.gvfFrame.mu.route( dn["GVFFILTER"].mu )
PSTK.gvfFrame.outputRGBA.route( dn["GVFFILTER"].outputRGBA )
PSTK.gvfFrame.execute.route( dn["GVFFILTER"].executeFilter )

# Sigmoid routings
PSTK.sigmoidFrame.input.menuIndex.route( dn["SIGMOIDFILTER"].inputIndex )
PSTK.sigmoidFrame.output.menuIndex.route( dn["SIGMOIDFILTER"].outputIndex )
PSTK.sigmoidFrame.alpha.route( dn["SIGMOIDFILTER"].alpha )
PSTK.sigmoidFrame.beta.route( dn["SIGMOIDFILTER"].beta )
PSTK.sigmoidFrame.execute.route( dn["SIGMOIDFILTER"].executeFilter )

# Bilateral routings
PSTK.bilateralFrame.input.menuIndex.route( dn["BILATERALFILTER"].inputIndex )
PSTK.bilateralFrame.output.menuIndex.route( dn["BILATERALFILTER"].outputIndex )
PSTK.bilateralFrame.sigmaD.route( dn["BILATERALFILTER"].sigmaSpatial )
PSTK.bilateralFrame.sigmaR.route( dn["BILATERALFILTER"].sigmaRange )
PSTK.bilateralFrame.execute.route( dn["BILATERALFILTER"].executeFilter )

# Fuzzy connectedness routings
PSTK.fuzzyFrame.input.menuIndex.route( dn["FCFILTER"].inputIndex )
PSTK.fuzzyFrame.output.menuIndex.route( dn["FCFILTER"].outputIndex )
PSTK.fuzzyFrame.seed.menuIndex.route( dn["FCFILTER"].seedIndex )
PSTK.fuzzyFrame.threshold.route( dn["FCFILTER"].fcThreshold )
PSTK.fuzzyFrame.inputIsAffinity.route( dn["FCFILTER"].inputIsAffinity )
PSTK.fuzzyFrame.invertAffinity.route( dn["FCFILTER"].invertAffinity )
PSTK.fuzzyFrame.execute.route( dn["FCFILTER"].executeFilter )

# Fast marching routings
PSTK.fastMarchingFrame.input.menuIndex.route( dn["FMFILTER"].inputIndex )
PSTK.fastMarchingFrame.output.menuIndex.route( dn["FMFILTER"].outputIndex )
PSTK.fastMarchingFrame.seed.menuIndex.route( dn["FMFILTER"].seedIndex )
PSTK.fastMarchingFrame.timeThreshold.route( dn["FMFILTER"].timeThreshold )
PSTK.fastMarchingFrame.normalizeOutput.route( dn["FMFILTER"].normalizeOutput )
PSTK.fastMarchingFrame.execute.route( dn["FMFILTER"].executeFilter )

# Potential routings
PSTK.potentialFrame.input.menuIndex.route( dn["POTENTIALFILTER"].inputIndex )
PSTK.potentialFrame.output.menuIndex.route( dn["POTENTIALFILTER"].outputIndex )
PSTK.potentialFrame.execute.route( dn["POTENTIALFILTER"].executeFilter )

# Potential routings
PSTK.distanceMapFrame.input.menuIndex.route( dn["DISTANCEMAPFILTER"].inputIndex )
PSTK.distanceMapFrame.output.menuIndex.route( dn["DISTANCEMAPFILTER"].outputIndex )
PSTK.distanceMapFrame.execute.route( dn["DISTANCEMAPFILTER"].executeFilter )
PSTK.distanceMapFrame.fromBackground.route( dn["DISTANCEMAPFILTER"].fromBackground )

# CompareSegmentations routings
PSTK.compareFrame.trueDelineation.menuIndex.route( dn["COMPARE"].trueDelineationIndex )
PSTK.compareFrame.delineation.menuIndex.route( dn["COMPARE"].delineationIndex )
PSTK.compareFrame.execute.route( dn["COMPARE"].compare )
dn["COMPARE"].compare.route( PSTK.compareFrame.infostring )

# VolumeRenderer routings
PSTK.volumeRendererFrame.volume.menuIndex.route( dn["VOLUMETEXTURE"].imageIndex )
PSTK.volumeRendererFrame.normals.menuIndex.route( dn["NORMALTEXTURE"].imageIndex )
PSTK.volumeRendererFrame.renderMode.route( dn["VOLUMERENDERER"].renderMode )
PSTK.volumeRendererFrame.isoValue.route( dn["VOLUMERENDERER"].isoValue )
PSTK.volumeRendererFrame.rayStep.route( dn["VOLUMERENDERER"].rayStep )
PSTK.volumeRendererFrame.active.route( dn["VOLUMERENDERER"].active );

PSTK.volumeRendererFrame.redAlpha.route( dn["REDTF"].alpha )
PSTK.volumeRendererFrame.redBeta.route( dn["REDTF"].beta )
PSTK.volumeRendererFrame.greenAlpha.route( dn["GREENTF"].alpha )
PSTK.volumeRendererFrame.greenBeta.route( dn["GREENTF"].beta )
PSTK.volumeRendererFrame.blueAlpha.route( dn["BLUETF"].alpha )
PSTK.volumeRendererFrame.blueBeta.route( dn["BLUETF"].beta )
PSTK.volumeRendererFrame.alphaAlpha.route( dn["ALPHATF"].alpha )
PSTK.volumeRendererFrame.alphaBeta.route( dn["ALPHATF"].beta )

# Deformable
PSTK.meshFrame.meshOpen.route( dn["MESH"].readMesh )
PSTK.meshFrame.meshSave.route( dn["MESH"].writeMesh )
PSTK.meshFrame.output.menuIndex.route( dn["MESH"].outputIndex )
PSTK.meshFrame.meshRasterize.route( dn["MESH"].rasterizeMesh )
PSTK.meshFrame.resetButton.route( dn["MESH"].resetButton )
PSTK.meshFrame.refineButton.route( dn["MESH"].refineButton )
PSTK.meshFrame.activateButton.route( dn["MESH"].activateButton )
PSTK.meshFrame.gradmag.menuIndex.route( dn["MESH"].gradientMagnitudeIndex )
PSTK.meshFrame.gmWeight.route( dn["MESH"].gradientMagnitudeWeight )
PSTK.meshFrame.potential.menuIndex.route( dn["MESH"].potentialIndex )
PSTK.meshFrame.potentialWeight.route( dn["MESH"].potentialWeight )
PSTK.meshFrame.propagation.menuIndex.route( dn["MESH"].propagationIndex )
PSTK.meshFrame.propagationWeight.route( dn["MESH"].propagationWeight )
PSTK.meshFrame.propagationZeroLevel.route( dn["MESH"].propagationZeroLevel )
PSTK.meshFrame.vector.menuIndex.route( dn["MESH"].vectorIndex )
PSTK.meshFrame.vectorWeight.route( dn["MESH"].vectorWeight )
PSTK.meshFrame.interactionSize.route( dn["MESH"].interactionSize )
PSTK.meshFrame.interactionWeight.route( dn["MESH"].interactionWeight )
PSTK.meshFrame.renderMode.route( dn["MESH"].renderMode )
PSTK.meshFrame.scaling.route( dn["MESH"].scaling )
PSTK.meshFrame.spatial.route( dn["MESH"].spatialWeight )
PSTK.meshFrame.external.route( dn["MESH"].externalWeight )

swb.button3.route( dn["MESH"].translationButton )
swb.button4.route( dn["MESH"].interactionButton )
#phantom.secondaryButton.route( dn["MESH"].interactionButton )
dn["VOLUME"].proxyPosition.route( dn["MESH"].translation )

# Live marching
PSTK.liveMarchingFrame.image.menuIndex.route( dn["LIVEMARCHING"].imageIndex )
PSTK.liveMarchingFrame.output.menuIndex.route( dn["LIVEMARCHING"].outputIndex )
PSTK.liveMarchingFrame.clearPoints.route( dn["LIVEMARCHING"].clearButton )
PSTK.liveMarchingFrame.rasterizeButton.route( dn["LIVEMARCHING"].rasterizeButton )
PSTK.liveMarchingFrame.active.route( dn["LIVEMARCHING"].active )
PSTK.liveMarchingFrame.minBacktrackDistance.route( dn["LIVEMARCHING"].minBacktrackDistance ) 
dn["SLICER"].sliceTranslation.route( dn["LIVEMARCHING"].sliceTranslation )
swb.button8.route( dn["LIVEMARCHING"].interactionButton )
swb.button7.route( dn["LIVEMARCHING"].addButton )
swb.button6.route( dn["LIVEMARCHING"].deleteButton )
swb.button5.route( dn["LIVEMARCHING"].closeButton )
dn["LIVEMARCHING"].closeButton.route( dn["OVERLAY"].updateTextureField )

# class for displaying the position and value as text
class PositionAndValueText(TypedField(MFString, (SFVec3f, SFFloat, SFFloat))):
  def update( self, event ):
    pos  = self.getRoutesIn()[0].getValue()
    valn = self.getRoutesIn()[1].getValue()
    val  = self.getRoutesIn()[2].getValue()
    return ["Position: (%7.2f, %7.2f, %7.2f)"%(pos.x, pos.y,pos.z),
            "Value: %7.2f (%7.2f)"%(valn,val)]

# class for displaying the image label as text
class ImageLabelText(TypedField(MFString, (SFInt32))):
  def update( self, event ):
    ind = event.getValue()
    s = "No volume loaded"
    if ind>=0:
        s = dn["VOLUME"].imageLabels.getValue()[ind]
    return [s]

posvaltext = PositionAndValueText()
imagetext = ImageLabelText()

dn["VOLUME"].currentImage.route( imagetext )
imagetext.route( dn["IMAGE_TEXT"].string )

dn["VOLUME"].voxelPosition.route( posvaltext )
dn["VOLUME"].voxelValueNormalized.route( posvaltext )
dn["VOLUME"].voxelValue.route( posvaltext )
posvaltext.route( dn["POSVAL_TEXT"].string )
