#
#
# Copyright 2007, Erik Vidholm
#
#
#   This file is part of WISH.
#
#   WISH is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   WISH is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with WISH.  If not, see <http:#www.gnu.org/licenses/>.
#
#

from H3D import *
from H3DInterface import *

import Tkinter
import tkMessageBox
import tkFileDialog

import math
import sys
import os

# --- File ---
# Tkinter frame for file actions such as Open, Save, and Quit
class FileFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)

        self.initopendir = "../../data"
        self.initsavedir = "../../data"
        
        self["text"] = "File"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.fileOpen = SFString()
        self.fileOpen.setValue("")
        self.fileNew = SFBool()
        self.fileNew.setValue(False)
        self.fileSave = SFString()
        self.fileSave.setValue("")
        self.fileClear = SFBool()
        self.fileClear.setValue(False)
        self.fileQuit = SFBool()
        self.fileQuit.setValue(False)

        self.width = SFInt32()
        self.width.setValue(128)
        self.height = SFInt32()
        self.height.setValue(128)
        self.depth = SFInt32()
        self.depth.setValue(128)

        self.spacingX = SFFloat()
        self.spacingX.setValue(1.0)
        self.spacingY = SFFloat()
        self.spacingY.setValue(1.0)
        self.spacingZ = SFFloat()
        self.spacingZ.setValue(1.0)
         
        self.createWidgets()
        
    def createWidgets(self):
        currentrow = 0

        open_button = Tkinter.Button(self,\
                                     command=self.openfunc,\
                                     text="Open")
        open_button.config(width=10)
        open_button.grid(row=0, column=0, sticky="ew")
        
        new_button = Tkinter.Button(self,\
                                    command=lambda : self.fileNew.setValue( True ),\
                                    text="New")
        new_button.config(width=10)
        new_button.grid(row=0, column=1, sticky="ew")
        
        save_button = Tkinter.Button(self,\
                                     command=self.savefunc,\
                                     text="Save")
        save_button.config(width=10)
        save_button.grid(row=0, column=2, sticky="ew")

        clear_button = Tkinter.Button(self,\
                                      command=self.clearfunc,\
                                      text="Clear")
        clear_button.config(width=10)
        clear_button.grid(row=0, column=3, sticky="ew")
        
        quit_button = Tkinter.Button(self,\
                                     command=self.quitfunc,\
                                     text="Quit")
        quit_button.config(width=10)
        quit_button.grid(row=0, column=4, sticky="ew")

        widthframe = Tkinter.Frame(self)
        Tkinter.Label(widthframe, text="Width").pack(anchor="w")
        self.width_var = Tkinter.StringVar()
        self.width_var.set("128")
        self.width_var.trace_variable("w", self.widthfunc)        
        widthspinner = Tkinter.Spinbox(widthframe,\
                                       from_=1, to=1024,\
                                       command=self.widthfunc,\
                                       textvariable=self.width_var)
        widthspinner.config(width=10)
        widthspinner.pack()
        widthframe.grid(row=1, column=0)

        heightframe = Tkinter.Frame(self)
        Tkinter.Label(heightframe, text="Height").pack(anchor="w")
        self.height_var = Tkinter.StringVar()
        self.height_var.set("128")
        self.height_var.trace_variable("w", self.heightfunc)        
        heightspinner = Tkinter.Spinbox(heightframe,\
                                        from_=1, to=1024,\
                                        command=self.heightfunc,\
                                        textvariable=self.height_var)
        heightspinner.config(width=10)
        heightspinner.pack()
        heightframe.grid(row=1, column=1)
        
        depthframe = Tkinter.Frame(self)
        Tkinter.Label(depthframe, text="Depth").pack(anchor="w")
        self.depth_var = Tkinter.StringVar()
        self.depth_var.set("128")
        self.depth_var.trace_variable("w", self.depthfunc)        
        depthspinner = Tkinter.Spinbox(depthframe,\
                                       from_=1, to=1024,\
                                       command=self.depthfunc,\
                                       textvariable=self.depth_var)
        depthspinner.config(width=10)
        depthspinner.pack()
        depthframe.grid(row=1, column=2)

        sxframe = Tkinter.Frame(self)
        Tkinter.Label(sxframe, text="Sx").pack(anchor="w")
        self.sx_var = Tkinter.StringVar()
        self.sx_var.set("1.0")
        self.sx_var.trace_variable("w", self.sxfunc)        
        sxspinner = Tkinter.Spinbox(sxframe,\
                                    from_=0.0, to=10.0,\
                                    increment=0.01,\
                                    command=self.sxfunc,\
                                    textvariable=self.sx_var)
        sxspinner.config(width=10)
        sxspinner.pack()
        sxframe.grid(row=2, column=0)

        syframe = Tkinter.Frame(self)
        Tkinter.Label(syframe, text="Sy").pack(anchor="w")
        self.sy_var = Tkinter.StringVar()
        self.sy_var.set("1.0")
        self.sy_var.trace_variable("w", self.syfunc)        
        syspinner = Tkinter.Spinbox(syframe,\
                                    from_=0.0, to=10.0,\
                                    increment=0.01,\
                                    command=self.syfunc,\
                                    textvariable=self.sy_var)
        syspinner.config(width=10)
        syspinner.pack()
        syframe.grid(row=2, column=1)

        szframe = Tkinter.Frame(self)
        Tkinter.Label(szframe, text="Sz").pack(anchor="w")
        self.sz_var = Tkinter.StringVar()
        self.sz_var.set("1.0")
        self.sz_var.trace_variable("w", self.szfunc)        
        szspinner = Tkinter.Spinbox(szframe,\
                                    from_=0.0, to=10.0,\
                                    increment=0.01,\
                                    command=self.szfunc,\
                                    textvariable=self.sz_var)
        szspinner.config(width=10)
        szspinner.pack()
        szframe.grid(row=2, column=2)

        # create the canvas, size in pixels
        canvas = Tkinter.Canvas(self, width=160, height=65) 
        canvas.grid(row=1, column=3, rowspan=2, columnspan=2, pady=2, padx=2)

        # load the image
        imfile = os.environ.get("WISH_ROOT") + "/wishH3D/demo/wish.ppm"
        self.im = Tkinter.PhotoImage(file=imfile)
        
        # put image on canvas
        canvas.create_image(8, 6, image=self.im, anchor="nw")
        
    def widthfunc( self, *args ):
        s = self.width_var.get()
        if not s=="":
            self.width.setValue( int(s) )

    def heightfunc( self, *args ):
        s = self.height_var.get()
        if not s=="":
            self.height.setValue( int(s) )

    def depthfunc( self, *args ):
        s = self.depth_var.get()
        if not s=="":
            self.depth.setValue( int(s) )

    def sxfunc( self, *args ):
        s = self.sx_var.get()
        if not s=="":
            self.spacingX.setValue( float(s) )

    def syfunc( self, *args ):
        s = self.sy_var.get()
        if not s=="":
            self.spacingY.setValue( float(s) )

    def szfunc( self, *args ):
        s = self.sz_var.get()
        if not s=="":
            self.spacingZ.setValue( float(s) )
        
    def openfunc( self ):
        fnms = tkFileDialog.askopenfilenames(initialdir=self.initopendir)
        if not fnms==[]:
            for f in fnms:
                if not f=="":
                    self.fileOpen.setValue( f )
            if not fnms[0]=="":        
                self.initopendir = os.path.split( fnms[0] )[0]
        
    def clearfunc( self ):
        t = tkMessageBox.askyesno("Clear all images", "Are you sure?")
        if t:
            self.fileClear.setValue( True )
        
    def savefunc( self ):
        fnm = tkFileDialog.asksaveasfilename(initialdir=self.initsavedir)
        if not fnm=="":
            self.fileSave.setValue( fnm )
            self.initsavedir = os.path.split( fnm )[0]
            
    def quitfunc( self ):
        t = tkMessageBox.askyesno("Quit", "Are you sure?")
        if t:
            self.fileQuit.setValue( True )
                



# the image labels from the volume
class ImageLabels( AutoUpdate(MFString) ):
    def update(self,event):
        refreshMenus( event.getValue() )
        return event.getValue()

imageLabels = ImageLabels()

class ImageMenu(Tkinter.Frame):
    def __init__(self, master=None, text="ImageMenu"):
        Tkinter.Frame.__init__(self, master)
        self.var = Tkinter.StringVar()
        self.var.set("None")
        self.var.trace_variable("w", self.varfunc)
        self.menuIndex = SFInt32()
        self.menuIndex.setValue(-1)

        # Label
        self.label = Tkinter.Label(self, text=text)
        self.label.pack(anchor="w")
        
        # OptionMenu
        self.optionmenu = Tkinter.OptionMenu(self, self.var, "None")
        self.optionmenu.config(width=10, anchor="w")
        self.optionmenu.pack()
        
    def insertItem(self, item):
        self.optionmenu["menu"].insert("end",\
                                       "command",\
                                       label=item,\
                                       command=lambda : self.var.set(item))
        
    def deleteItems( self ):
        self.optionmenu["menu"].delete(0,"end")
    
    def updateMenu(self, labels):
        self.deleteItems()
        i = 0
        for item in labels:
            i=i+1
            self.insertItem( item )

        self.insertItem("None")
        
    def selectItem( self, index ):
        if index>=0:
            self.optionmenu["menu"].invoke( index )
        else:
            ind = self.optionmenu["menu"].index( "None" )
            self.optionmenu["menu"].invoke( ind )
            
    def varfunc( self, name, index, mode ):
        ind = -1
        if not self.var.get() == "None":
            ind = self.optionmenu["menu"].index( self.var.get() )
        self.menuIndex.setValue( ind )


# --- Volume ---
# Tkinter frame for volume actions
class VolumeFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Volume slicer"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.renderMode = SFInt32()
        self.renderMode.setValue(0)
        self.desiredSize = SFFloat()
        self.desiredSize.setValue(0.2)
        self.window = SFFloat()
        self.window.setValue(1.0)
        self.level = SFFloat()
        self.level.setValue(0.5)

        self.activeX = SFBool()
        self.activeX.setValue( True )
        self.activeY = SFBool()
        self.activeY.setValue( True )
        self.activeZ = SFBool()
        self.activeZ.setValue( True )
        
        self.hapticButton = SFBool()
        self.hapticButton.setValue( False )

        self.minval = 0.0
        self.maxval = 1.0
        
        self.createWidgets()
                
    def refresh(self):
        vthis = self.current.menuIndex.getValue()
        vroute = self.current.menuIndex.getRoutesOut()[0].getValue()
        if not vthis==vroute:
            self.current.selectItem( vroute )

    def refreshMenus(self, labels):
        self.current.updateMenu(labels)
        self.secondary.updateMenu(labels)
        self.overlay.updateMenu(labels)
        
    def createWidgets(self):
        self.current = ImageMenu(self, text="Current image")
        self.current.grid(row=0, column=0) 

        self.secondary = ImageMenu(self, text="Secondary")
        self.secondary.grid(row=1, column=0) 

        self.overlay = ImageMenu(self, text="Overlay")
        self.overlay.grid(row=2, column=0) 
        
        rendermodeframe = Tkinter.Frame(self)
        Tkinter.Label(rendermodeframe, text="Render mode").pack(anchor="w")
        self.render_var = Tkinter.StringVar()
        self.render_var.set("Standard")
        self.render_var.trace_variable("w", self.render_var_func)        
        om = Tkinter.OptionMenu(rendermodeframe, self.render_var, "Standard", "Region", "Colormap", "Color")
        om.config(width=10)
        om.pack()
        rendermodeframe.grid(row=3, column=0)

        desiredscale = Tkinter.Scale(self,\
                                     command=lambda v: self.desiredSize.setValue( float(v) ),\
                                     from_=0.0,\
                                     to=1.0,\
                                     resolution=0.01,\
                                     label="Size",\
                                     orient="horizontal")
        desiredscale.grid(row=0, column=1, sticky="news")
        desiredscale.set( self.desiredSize.getValue() )
        
        windowscale = Tkinter.Scale(self,\
                                    command=lambda v: self.window.setValue( float(v)*(self.maxval-self.minval) ),\
                                    from_=0.0,\
                                    to=1.0,\
                                    resolution=0.001,\
                                    label="Window",\
                                    orient="horizontal")
        windowscale.grid(row=1, column=1, sticky="news")
        windowscale.set( self.window.getValue() )
        
        levelscale = Tkinter.Scale(self,\
                                   command=lambda v: self.level.setValue( self.minval + float(v)*self.maxval ),\
                                   from_=0.0,\
                                   to=1.0,\
                                   resolution=0.001,\
                                   label="Level",\
                                   orient="horizontal")
        levelscale.grid(row=2, column=1, sticky="news")
        levelscale.set( self.level.getValue() )

        checkframe = Tkinter.Frame(self)
        
        self.activex_var = Tkinter.IntVar()
        self.activex_var.set( 1 )
        activex_button = Tkinter.Checkbutton(checkframe,\
                                            text="x",\
                                            variable=self.activex_var,\
                                            command=lambda : self.activeX.setValue(self.activex_var.get()) )
        activex_button.grid(row=0, column=0, sticky="w")

        self.activey_var = Tkinter.IntVar()
        self.activey_var.set( 1 )
        activey_button = Tkinter.Checkbutton(checkframe,\
                                            text="y",\
                                            variable=self.activey_var,\
                                            command=lambda : self.activeY.setValue(self.activey_var.get()) )
        activey_button.grid(row=0, column=1, sticky="w", padx=2)

        self.activez_var = Tkinter.IntVar()
        self.activez_var.set( 1 )
        activez_button = Tkinter.Checkbutton(checkframe,\
                                            text="z",\
                                            variable=self.activez_var,\
                                            command=lambda : self.activeZ.setValue(self.activez_var.get()) )
        activez_button.grid(row=0, column=2, sticky="w", padx=2)
        
        self.haptic_var = Tkinter.IntVar()
        self.haptic_var.set( 0 )
        haptic_button = Tkinter.Checkbutton(checkframe,\
                                            text="Haptics",\
                                            variable=self.haptic_var,\
                                            command=lambda : self.hapticButton.setValue(self.haptic_var.get()) )
        haptic_button.grid(row=0, column=3, sticky="w", padx=2)

        checkframe.grid(row=4, column=0, columnspan=2, sticky="w", pady=5)

        minmaxframe = Tkinter.Frame(self)
        
        rangelabel = Tkinter.Label(minmaxframe, text="Window/Level range")
        rangelabel.grid(row=0, column=0, columnspan=2, sticky="w")

        minframe = Tkinter.Frame(minmaxframe)
        self.min_var = Tkinter.StringVar()
        self.min_var.set("0.0")
        self.min_var.trace_variable("w", self.minfunc)        
        minspinner = Tkinter.Spinbox(minframe,\
                                     from_=0.0, to=1.0,\
                                     increment=0.01,\
                                     command=self.minfunc,\
                                     textvariable=self.min_var)
        minspinner.config(width=8)
        minspinner.pack()
        minframe.grid(row=1, column=0, padx=5)

        maxframe = Tkinter.Frame(minmaxframe)
        self.max_var = Tkinter.StringVar()
        self.max_var.set("1.0")
        self.max_var.trace_variable("w", self.maxfunc)        
        maxspinner = Tkinter.Spinbox(maxframe,\
                                     from_=0.0, to=1.0,\
                                     increment=0.01,\
                                     command=self.maxfunc,\
                                     textvariable=self.max_var)
        maxspinner.config(width=8)
        maxspinner.pack()
        maxframe.grid(row=1, column=1, padx=5)

        minmaxframe.grid(row=3, column=1)
        
    def render_var_func( self, name, index, mode ):
        v = self.render_var.get()
        if v=="Standard":
            self.renderMode.setValue(0)
        elif v=="Region":
            self.renderMode.setValue(1)
        elif v=="Colormap":
            self.renderMode.setValue(2)
        elif v=="Color":
            self.renderMode.setValue(3)

    def minfunc( self, *args ):
        s = self.min_var.get()
        if not s=="":
            self.minval = float(s)

    def maxfunc( self, *args ):
        s = self.max_var.get()
        if not s=="":
            self.maxval = float(s)
            
# --- EditTexture ---
class EditTextureFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "EditTexture"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.clearAll = SFBool()
        self.clearLabel = SFBool()
        self.editLabel = SFInt32()
        self.editLabel.setValue( 1 )
        self.editSize = SFFloat()
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.image.updateMenu(labels)
        
    def createWidgets(self):
        self.image = ImageMenu(self, text="Image")
        self.image.pack(side="top")
        
        edit_size = Tkinter.Scale(self,\
                                  command=lambda v: self.editSize.setValue( float(v) ),\
                                  from_=1.0,\
                                  to=10.0,\
                                  resolution=0.01,\
                                  label="Edit size",\
                                  orient="horizontal")
        edit_size.pack(side="top")
        edit_size.set( self.editSize.getValue() )

        labelframe = Tkinter.Frame(self)
        Tkinter.Label(labelframe, text="Label").pack(anchor="w")
        self.label_var = Tkinter.StringVar()
        self.label_var.set("1")
        self.label_var.trace_variable("w", self.labelfunc)        
        labelspinner = Tkinter.Spinbox(labelframe,\
                                       from_=1, to=255,\
                                       command=self.labelfunc,\
                                       textvariable=self.label_var)
        labelspinner.config(width=15)
        labelspinner.pack()
        labelframe.pack(side="top", pady=5)
        
        clearall_button = Tkinter.Button(self,\
                                         command=lambda : self.clearAll.setValue( True ),\
                                         text="Clear all")
        clearall_button.config(width=15)
        clearall_button.pack(side="top")

        clearlabel_button = Tkinter.Button(self,\
                                           command=lambda : self.clearLabel.setValue( True ),\
                                           text="Clear label")
        clearlabel_button.config(width=15)
        clearlabel_button.pack(side="top")

        helptext = Tkinter.Label(self, justify="left", bg="gray", text="Button 1: Draw\nButton 2: Erase")
        helptext.pack(side="top", padx=10, pady=10)
        
    def labelfunc( self, *args ):
        s = self.label_var.get()
        if not s=="":
            lbl = int(s)
            if lbl>0 and lbl<=255:
                self.editLabel.setValue( lbl )




# --- Haptics frame ---
class HapticsFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Volume haptics"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.hapticMode = SFInt32()
        self.hapticMode.setValue(0)

        self.activeButton = SFBool()
        self.activeButton.setValue( False )

        self.flipDirection = SFBool()
        self.flipDirection.setValue( True )

        self.onlyXY = SFBool()
        self.onlyXY.setValue( False )
        
        self.viscosityAlpha = SFFloat()
        self.viscosityAlpha.setValue(0.1)
        self.viscosityBeta = SFFloat()
        self.viscosityBeta.setValue(0.5)

        self.strength = SFFloat()
        self.strength.setValue(2.5)

        self.friction = SFFloat()
        self.friction.setValue(0.5)

        self.distinctnessAlpha = SFFloat()
        self.distinctnessAlpha.setValue(0.1)
        self.distinctnessBeta = SFFloat()
        self.distinctnessBeta.setValue(0.2)

        self.strengthAlpha = SFFloat()
        self.strengthAlpha.setValue(0.1)
        self.strengthBeta = SFFloat()
        self.strengthBeta.setValue(0.1)

        self.stiffness = SFFloat()
        self.stiffness.setValue( 500.0 )
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.image.updateMenu(labels)
        
    def createWidgets(self):
        self.image = ImageMenu(self, text="Image")
        self.image.grid(row=0, column=0) 

        hapticmodeframe = Tkinter.Frame(self)
        Tkinter.Label(hapticmodeframe, text="Mode").pack(anchor="w")
        self.mode_var = Tkinter.StringVar()
        self.mode_var.set("Surface")
        self.mode_var.trace_variable("w", self.mode_var_func)        
        om = Tkinter.OptionMenu(hapticmodeframe, self.mode_var, "Surface", "Viscosity", "Potential", "Vector")
        om.config(width=10)
        om.pack()
        hapticmodeframe.grid(row=1, column=0)

        checkbuttonframe = Tkinter.Frame(self)
        
        self.active_var = Tkinter.IntVar()
        self.active_var.set( 0 )
        active_button = Tkinter.Checkbutton(checkbuttonframe,\
                                            text="Active",\
                                            variable=self.active_var,\
                                            command=lambda : self.activeButton.setValue(self.active_var.get()) )
        active_button.grid(row=0,column=0,sticky="w")
        
        self.flip_var = Tkinter.IntVar()
        self.flip_var.set( 1 )
        flip_button = Tkinter.Checkbutton(checkbuttonframe,\
                                          text="Flip direction",\
                                          variable=self.flip_var,\
                                          command=lambda : self.flipDirection.setValue(self.flip_var.get()) )
        flip_button.grid(row=1,column=0,sticky="w")

        self.xy_var = Tkinter.IntVar()
        self.xy_var.set( 0 )
        xy_button = Tkinter.Checkbutton(checkbuttonframe,\
                                        text="Only XY",\
                                        variable=self.xy_var,\
                                        command=lambda : self.onlyXY.setValue(self.xy_var.get()) )
        xy_button.grid(row=2,column=0,sticky="w")

        checkbuttonframe.grid(row=3, column=0, sticky="n")

        stiffness_scale = Tkinter.Scale(self,\
                                             command=lambda v: self.stiffness.setValue(float(v)),\
                                             from_=100.0,\
                                             to=800.0,\
                                             resolution=0.1,\
                                             label="Stiffness",\
                                             orient="horizontal")
        stiffness_scale.grid(row=2, column=0)
        stiffness_scale.set( self.stiffness.getValue() )
        
        viscosityalpha_scale = Tkinter.Scale(self,\
                                             command=lambda v: self.viscosityAlpha.setValue(float(v)),\
                                             from_=0.0,\
                                             to=1.0,\
                                             resolution=0.01,\
                                             label="Viscosity alpha",\
                                             orient="horizontal")
        viscosityalpha_scale.grid(row=0, column=1)
        viscosityalpha_scale.set( self.viscosityAlpha.getValue() )
        
        viscositybeta_scale = Tkinter.Scale(self,\
                                            command=lambda v: self.viscosityBeta.setValue(float(v)),\
                                            from_=0.0,\
                                            to=1.0,\
                                            resolution=0.01,\
                                            label="Viscosity beta",\
                                            orient="horizontal")
        viscositybeta_scale.grid(row=0, column=2)
        viscositybeta_scale.set( self.viscosityBeta.getValue() )

        strength_scale = Tkinter.Scale(self,\
                                       command=lambda v: self.strength.setValue(float(v)),\
                                       from_=0.0,\
                                       to=5.0,\
                                       resolution=0.01,\
                                       label="Strength",\
                                       orient="horizontal")
        strength_scale.grid(row=1, column=1)
        strength_scale.set( self.strength.getValue() )
        
        friction_scale = Tkinter.Scale(self,\
                                       command=lambda v: self.friction.setValue(float(v)),\
                                       from_=0.0,\
                                       to=1.0,\
                                       resolution=0.01,\
                                       label="Friction",\
                                       orient="horizontal")
        friction_scale.grid(row=1, column=2)
        friction_scale.set( self.friction.getValue() )
        
        distinctnessalpha_scale = Tkinter.Scale(self,\
                                                command=lambda v: self.distinctnessAlpha.setValue(float(v)),\
                                                from_=0.0,\
                                                to=1.0,\
                                                resolution=0.01,\
                                                label="Distinctness alpha",\
                                                orient="horizontal")
        distinctnessalpha_scale.grid(row=2, column=1)
        distinctnessalpha_scale.set( self.distinctnessAlpha.getValue() )
        
        distinctnessbeta_scale = Tkinter.Scale(self,\
                                               command=lambda v: self.distinctnessBeta.setValue(float(v)),\
                                               from_=0.0,\
                                               to=5.0,\
                                               resolution=0.01,\
                                               label="Distinctness beta",\
                                               orient="horizontal")
        distinctnessbeta_scale.grid(row=2, column=2)
        distinctnessbeta_scale.set( self.distinctnessBeta.getValue() )

        strengthalpha_scale = Tkinter.Scale(self,\
                                            command=lambda v: self.strengthAlpha.setValue(float(v)),\
                                            from_=0.0,\
                                            to=1.0,\
                                            resolution=0.01,\
                                            label="Strength alpha",\
                                            orient="horizontal")
        strengthalpha_scale.grid(row=3, column=1)
        strengthalpha_scale.set( self.strengthAlpha.getValue() )
        
        strengthbeta_scale = Tkinter.Scale(self,\
                                           command=lambda v: self.strengthBeta.setValue(float(v)),\
                                           from_=0.0,\
                                           to=1.0,\
                                           resolution=0.01,\
                                           label="Strength beta",\
                                           orient="horizontal")
        strengthbeta_scale.grid(row=3, column=2)
        strengthbeta_scale.set( self.strengthBeta.getValue() )
        
    def mode_var_func( self, name, index, mode ):
        v = self.mode_var.get()
        if v=="Surface":
            self.hapticMode.setValue(0)
        elif v=="Viscosity":
            self.hapticMode.setValue(1)
        elif v=="Potential":
            self.hapticMode.setValue(2)
        elif v=="Vector":
            self.hapticMode.setValue(3)



# --- VolumeRenderer frame ---
class VolumeRendererFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Volume rendering"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.renderMode = SFInt32()
        self.renderMode.setValue(0)
        self.isoValue = SFFloat()
        self.isoValue.setValue(0.5)
        self.redAlpha = SFFloat()
        self.redAlpha.setValue(0.1)
        self.redBeta = SFFloat()
        self.redBeta.setValue(0.5)
        self.greenAlpha = SFFloat()
        self.greenAlpha.setValue(0.1)
        self.greenBeta = SFFloat()
        self.greenBeta.setValue(0.5)
        self.blueAlpha = SFFloat()
        self.blueAlpha.setValue(0.1)
        self.blueBeta = SFFloat()
        self.blueBeta.setValue(0.5)
        self.alphaAlpha = SFFloat()
        self.alphaAlpha.setValue(0.1)
        self.alphaBeta = SFFloat()
        self.alphaBeta.setValue(0.5)
        self.rayStep = SFFloat()
        self.rayStep.setValue(0.005)
        
        self.active = SFBool()
        self.active.setValue( False )

        self.minval = 0.0
        self.maxval = 0.0
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.volume.updateMenu(labels)
        self.normals.updateMenu(labels)
        
    def createWidgets(self):
        self.volume = ImageMenu(self, text="Volume")
        self.volume.grid(row=0, column=0)
        
        self.normals = ImageMenu(self, text="Normals")
        self.normals.grid(row=1, column=0)

        rendermodeframe = Tkinter.Frame(self)
        Tkinter.Label(rendermodeframe, text="Render mode").pack(anchor="w")
        self.rendermode_var = Tkinter.StringVar()
        self.rendermode_var.set("MIP")
        self.rendermode_var.trace_variable("w", self.rendermode_var_func)        
        om = Tkinter.OptionMenu(rendermodeframe, self.rendermode_var, "MIP", "Composite", "Isosurface", "HSV-MIP")
        om.config(width=10)
        om.pack()
        rendermodeframe.grid(row=2, column=0)
        
        iso_scale = Tkinter.Scale(self,\
                                  command=lambda v: self.isoValue.setValue(self.minval + float(v)*self.maxval),\
                                  from_=0.0,\
                                  to=1.0,\
                                  resolution=0.001,\
                                  label="Isovalue",\
                                  orient="horizontal")
        iso_scale.grid(row=3, column=0)
        iso_scale.set( self.isoValue.getValue() )



        redalpha_scale = Tkinter.Scale(self,\
                                       command=lambda v: self.redAlpha.setValue(float(v)*(self.maxval-self.minval)),\
                                       from_=0.0,\
                                       to=1.0,\
                                       resolution=0.001,\
                                       label="Red alpha",\
                                       orient="horizontal")
        redalpha_scale.grid(row=0, column=1)
        redalpha_scale.set( self.redAlpha.getValue() )

        redbeta_scale = Tkinter.Scale(self,\
                                      command=lambda v: self.redBeta.setValue(self.minval + float(v)*self.maxval),\
                                      from_=0.0,\
                                      to=1.0,\
                                      resolution=0.001,\
                                      label="Red beta",\
                                      orient="horizontal")
        redbeta_scale.grid(row=0, column=2)
        redbeta_scale.set( self.redBeta.getValue() )

        greenalpha_scale = Tkinter.Scale(self,\
                                         command=lambda v: self.greenAlpha.setValue(float(v)*(self.maxval-self.minval)),\
                                         from_=0.0,\
                                         to=1.0,\
                                         resolution=0.001,\
                                         label="Green alpha",\
                                         orient="horizontal")
        greenalpha_scale.grid(row=1, column=1)
        greenalpha_scale.set( self.greenAlpha.getValue() )

        greenbeta_scale = Tkinter.Scale(self,\
                                        command=lambda v: self.greenBeta.setValue(self.minval + float(v)*self.maxval),\
                                        from_=0.0,\
                                        to=1.0,\
                                        resolution=0.001,\
                                        label="Green beta",\
                                        orient="horizontal")
        greenbeta_scale.grid(row=1, column=2)
        greenbeta_scale.set( self.greenBeta.getValue() )

        bluealpha_scale = Tkinter.Scale(self,\
                                        command=lambda v: self.blueAlpha.setValue(float(v)*(self.maxval-self.minval)),\
                                        from_=0.0,\
                                        to=1.0,\
                                        resolution=0.001,\
                                        label="Blue alpha",\
                                        orient="horizontal")
        bluealpha_scale.grid(row=2, column=1)
        bluealpha_scale.set( self.blueAlpha.getValue() )

        bluebeta_scale = Tkinter.Scale(self,\
                                       command=lambda v: self.blueBeta.setValue(self.minval + float(v)*self.maxval),\
                                       from_=0.0,\
                                       to=1.0,\
                                       resolution=0.001,\
                                       label="Blue beta",\
                                       orient="horizontal")
        bluebeta_scale.grid(row=2, column=2)
        bluebeta_scale.set( self.blueBeta.getValue() )

        alphaalpha_scale = Tkinter.Scale(self,\
                                         command=lambda v: self.alphaAlpha.setValue(float(v)*(self.maxval-self.minval)),\
                                         from_=0.0,\
                                         to=1.0,\
                                         resolution=0.001,\
                                         label="Alpha alpha",\
                                         orient="horizontal")
        alphaalpha_scale.grid(row=3, column=1)
        alphaalpha_scale.set( self.alphaAlpha.getValue() )

        alphabeta_scale = Tkinter.Scale(self,\
                                        command=lambda v: self.alphaBeta.setValue(self.minval + float(v)*self.maxval),\
                                        from_=0.0,\
                                        to=1.0,\
                                        resolution=0.001,\
                                        label="Alpha beta",\
                                        orient="horizontal")
        alphabeta_scale.grid(row=3, column=2)
        alphabeta_scale.set( self.alphaBeta.getValue() )
        
        raystep_scale = Tkinter.Scale(self,\
                                      command=lambda v: self.rayStep.setValue(float(v)),\
                                      from_=0.001,\
                                      to=0.5,\
                                      resolution=0.001,\
                                      label="Ray step",\
                                      orient="horizontal")
        raystep_scale.grid(row=4, column=0)
        raystep_scale.set( self.rayStep.getValue() )
        
        self.active_var = Tkinter.IntVar()
        self.active_var.set( 0 )
        active_button = Tkinter.Checkbutton(self,\
                                            text="Active",\
                                            variable=self.active_var,\
                                            command=lambda : self.active.setValue(self.active_var.get()) )
        active_button.grid(row=4,column=2,sticky="s")

        minmaxframe = Tkinter.Frame(self)
        
        rangelabel = Tkinter.Label(minmaxframe, text="Alpha/Beta range")
        rangelabel.grid(row=0, column=0, columnspan=2, sticky="w")

        minframe = Tkinter.Frame(minmaxframe)
        self.min_var = Tkinter.StringVar()
        self.min_var.set("0.0")
        self.min_var.trace_variable("w", self.minfunc)        
        minspinner = Tkinter.Spinbox(minframe,\
                                     from_=0.0, to=1.0,\
                                     increment=0.01,\
                                     command=self.minfunc,\
                                     textvariable=self.min_var)
        minspinner.config(width=8)
        minspinner.pack()
        minframe.grid(row=1, column=0, padx=5)

        maxframe = Tkinter.Frame(minmaxframe)
        self.max_var = Tkinter.StringVar()
        self.max_var.set("1.0")
        self.max_var.trace_variable("w", self.maxfunc)        
        maxspinner = Tkinter.Spinbox(maxframe,\
                                     from_=0.0, to=1.0,\
                                     increment=0.01,\
                                     command=self.maxfunc,\
                                     textvariable=self.max_var)
        maxspinner.config(width=8)
        maxspinner.pack()
        maxframe.grid(row=1, column=1, padx=5)

        minmaxframe.grid(row=4, column=1, sticky="s")


        
    def minfunc( self, *args ):
        s = self.min_var.get()
        if not s=="":
            self.minval = float(s)

    def maxfunc( self, *args ):
        s = self.max_var.get()
        if not s=="":
            self.maxval = float(s)

    def rendermode_var_func( self, name, index, mode ):
        v = self.rendermode_var.get()
        if v=="MIP":
            self.renderMode.setValue(0)
        elif v=="Composite":
            self.renderMode.setValue(1)
        elif v=="Isosurface":
            self.renderMode.setValue(2)
        elif v=="HSV-MIP":
            self.renderMode.setValue(3)

        
# --- Gaussian smoothing ---
class GSFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Gaussian smoothing"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.sigma = SFFloat()
        self.sigma.setValue(1.0)
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
        
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
                     
        sigma_scale = Tkinter.Scale(self,\
                                    command=lambda v: self.sigma.setValue(float(v)),\
                                    from_=0.1,\
                                    to=5.0,\
                                    resolution=0.01,\
                                    label="Sigma",\
                                    orient="horizontal")
        sigma_scale.pack()
        sigma_scale.set( self.sigma.getValue() )
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue(True),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)


# --- Gaussian gradient magnitude ---
class GGMFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Gaussian gradient magnitude"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.sigma = SFFloat()
        self.sigma.setValue(1.0)
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        sigma_scale = Tkinter.Scale(self,\
                                    command=lambda v: self.sigma.setValue( float(v) ),\
                                    from_=0.1,\
                                    to=5.0,\
                                    resolution=0.01,\
                                    label="Sigma",\
                                    orient="horizontal")
        sigma_scale.pack()
        sigma_scale.set( self.sigma.getValue() )
        
        execute_button = Tkinter.Button(self,\
                                             command=lambda : self.execute.setValue( True ),\
                                             text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)


# --- Gaussian gradient ---
class GGFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Gaussian gradient"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.sigma = SFFloat()
        self.sigma.setValue(1.0)
        self.outputRGBA = SFBool()
        self.outputRGBA.setValue( False )
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        sigma_scale = Tkinter.Scale(self,\
                                    command=lambda v: self.sigma.setValue( float(v) ),\
                                    from_=0.1,\
                                    to=5.0,\
                                    resolution=0.01,\
                                    label="Sigma",\
                                    orient="horizontal")
        sigma_scale.pack()
        sigma_scale.set( self.sigma.getValue() )

        self.outputrgba_var = Tkinter.IntVar()
        self.outputrgba_var.set( 0 )
        outputrgba_button = Tkinter.Checkbutton(self,\
                                                text="Output as RGBA",\
                                                variable=self.outputrgba_var,\
                                                command=lambda : self.outputRGBA.setValue(self.outputrgba_var.get()) )
        outputrgba_button.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)



# --- Gradient vector flow ---
class GVFFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "GVF"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.mu = SFFloat()
        self.mu.setValue(0.5)
        self.outputRGBA = SFBool()
        self.outputRGBA.setValue( False )
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Edge map")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        mu_scale = Tkinter.Scale(self,\
                                 command=lambda v: self.mu.setValue( float(v) ),\
                                 from_=0.1,\
                                 to=5.0,\
                                 resolution=0.01,\
                                 label="mu",\
                                 orient="horizontal")
        mu_scale.pack()
        mu_scale.set( self.mu.getValue() )

        self.outputrgba_var = Tkinter.IntVar()
        self.outputrgba_var.set( 0 )
        outputrgba_button = Tkinter.Checkbutton(self,\
                                                text="Output as RGBA",\
                                                variable=self.outputrgba_var,\
                                                command=lambda : self.outputRGBA.setValue(self.outputrgba_var.get()) )
        outputrgba_button.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)



# --- Sigmoid filter ---
class SigmoidFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Sigmoid"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.alpha = SFFloat()
        self.alpha.setValue(0.1)
        self.beta = SFFloat()
        self.beta.setValue(0.5)
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        alpha_scale = Tkinter.Scale(self,\
                                    command=lambda v: self.alpha.setValue( float(v) ),\
                                    from_=-1.0,\
                                    to=1.0,\
                                    resolution=0.01,\
                                    label="Alpha",\
                                    orient="horizontal")
        alpha_scale.pack()
        alpha_scale.set( self.alpha.getValue() )

        beta_scale = Tkinter.Scale(self,\
                                   command=lambda v: self.beta.setValue( float(v) ),\
                                   from_=0.0,\
                                   to=1.0,\
                                   resolution=0.01,\
                                   label="Beta",\
                                   orient="horizontal")
        beta_scale.pack()
        beta_scale.set( self.beta.getValue() )

        execute_button = Tkinter.Button(self,\
                                             command=lambda : self.execute.setValue( True ),\
                                             text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)
        
        

# --- Bilateral filter ---
class BilateralFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Bilateral"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields associated with the Bilateral frame
        self.sigmaD = SFFloat()
        self.sigmaD.setValue(1.0)
        self.sigmaR = SFFloat()
        self.sigmaR.setValue(0.1)
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        sigmad_scale = Tkinter.Scale(self,\
                                     command=lambda v: self.sigmaD.setValue( float(v) ),\
                                     from_=0.1,\
                                     to=5.0,\
                                     resolution=0.01,\
                                     label="Sigma domain",\
                                     orient="horizontal")
        sigmad_scale.pack()
        sigmad_scale.set( self.sigmaD.getValue() )
        
        sigmar_scale = Tkinter.Scale(self,\
                                     command=lambda v: self.sigmaR.setValue( float(v) ),\
                                     from_=0.01,\
                                     to=1.0,\
                                     resolution=0.01,\
                                     label="Sigma range",\
                                     orient="horizontal")
        sigmar_scale.pack()
        sigmar_scale.set( self.sigmaR.getValue() )

        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)
        

        
# --- Fuzzy connectedness filter ---
class FuzzyFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Fuzzy"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields associated with the Fuzzy connectedness frame
        self.threshold = SFFloat()
        self.threshold.setValue( 0.8 )

        self.inputIsAffinity = SFBool()
        self.inputIsAffinity.setValue( False )
        
        self.invertAffinity = SFBool()
        self.invertAffinity.setValue( False )
        
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
        self.seed.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Affinity")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        self.seed = ImageMenu(self, text="Seed")
        self.seed.pack()

        threshold_scale = Tkinter.Scale(self,\
                                        command=lambda v: self.threshold.setValue( float(v) ),\
                                        from_=0.0,\
                                        to=1.0,\
                                        resolution=0.01,\
                                        label="FC threshold",\
                                        orient="horizontal")
        threshold_scale.pack()
        threshold_scale.set( self.threshold.getValue() )

        self.inputisaffinity_var = Tkinter.IntVar()
        self.inputisaffinity_var.set( 0 )
        inputisaffinity_button = Tkinter.Checkbutton(self,\
                                                     text="Input is affinity",\
                                                     variable=self.inputisaffinity_var,\
                                                     command=lambda : self.inputIsAffinity.setValue(self.inputisaffinity_var.get()) )
        inputisaffinity_button.pack()

        self.invertaffinity_var = Tkinter.IntVar()
        self.invertaffinity_var.set( 0 )
        invertaffinity_button = Tkinter.Checkbutton(self,\
                                                    text="Invert affinity",\
                                                    variable=self.invertaffinity_var,\
                                                    command=lambda : self.invertAffinity.setValue(self.invertaffinity_var.get()) )
        invertaffinity_button.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)



# --- Fast marching filter ---
class FastMarchingFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Fast marching"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields associated with the Fuzzy connectedness frame
        self.timeThreshold = SFFloat()
        self.timeThreshold.setValue( 0.0 )

        self.normalizeOutput = SFBool()
        self.normalizeOutput.setValue( True )
        
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
        self.seed.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Cost")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        self.seed = ImageMenu(self, text="Seed")
        self.seed.pack()

        threshold_scale = Tkinter.Scale(self,\
                                        command=lambda v: self.timeThreshold.setValue( float(v)*100 ),\
                                        from_=0.0,\
                                        to=10000.0,\
                                        resolution=1,\
                                        label="Time threshold (100)",\
                                        orient="horizontal")
        threshold_scale.pack()
        threshold_scale.set( self.timeThreshold.getValue() )

        self.normalize_var = Tkinter.IntVar()
        self.normalize_var.set( 1 )
        normalize_button = Tkinter.Checkbutton(self,\
                                               text="Normalize output",\
                                               variable=self.normalize_var,\
                                               command=lambda : self.normalizeOutput.setValue(self.normalize_var.get()) )
        normalize_button.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)


# --- Potential filter ---
class PotentialFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Potential"
        self["padx"] = 5
        self["pady"] = 5
        
        # H3DApi fields associated with the Potential frame
        self.execute = SFBool()
        self.execute.setValue(False)
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                        
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)
        

# --- Distance map filter ---
class DistanceMapFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Distance map"
        self["padx"] = 5
        self["pady"] = 5
        
        # H3DApi fields associated with the Potential frame
        self.fromBackground = SFBool()
        self.fromBackground.setValue( True )

        self.execute = SFBool()
        self.execute.setValue(False)
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                        
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()

        self.frombg_var = Tkinter.IntVar()
        self.frombg_var.set( 1 )
        frombg_button = Tkinter.Checkbutton(self,\
                                            text="From background",\
                                            variable=self.frombg_var,\
                                            command=lambda : self.fromBackground.setValue(self.frombg_var.get()) )
        frombg_button.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)
        


# --- Threshold filter ---
class ThresholdFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Threshold"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.lowThreshold = SFFloat()
        self.lowThreshold.setValue( 0.0 )
        
        self.highThreshold = SFFloat()
        self.highThreshold.setValue( 255.0 )
        
        self.execute = SFBool()
        self.execute.setValue(False)

        self.createWidgets()

    def refreshMenus(self, labels):
        self.input.updateMenu(labels)
        self.output.updateMenu(labels)
                
    def createWidgets(self):

        self.input = ImageMenu(self, text="Input")
        self.input.pack()
        
        self.output = ImageMenu(self, text="Output")
        self.output.pack()

        ltframe = Tkinter.Frame(self)
        Tkinter.Label(ltframe, text="Low threshold").pack(anchor="w")
        self.lt_var = Tkinter.StringVar()
        self.lt_var.set( str(self.lowThreshold.getValue()) )
        self.lt_var.trace_variable("w", self.ltfunc)        
        ltspinner = Tkinter.Spinbox(ltframe,\
                                    from_=0.0, to=100000.0,\
                                    increment=1,\
                                    command=self.ltfunc,\
                                    textvariable=self.lt_var)
        ltspinner.config(width=10)
        ltspinner.pack()
        ltframe.pack()

        htframe = Tkinter.Frame(self)
        Tkinter.Label(htframe, text="High threshold").pack(anchor="w")
        self.ht_var = Tkinter.StringVar()
        self.ht_var.set( str(self.highThreshold.getValue()) )
        self.ht_var.trace_variable("w", self.htfunc)        
        htspinner = Tkinter.Spinbox(htframe,\
                                    from_=0.0, to=100000.0,\
                                    increment=1,\
                                    command=self.htfunc,\
                                    textvariable=self.ht_var)
        htspinner.config(width=10)
        htspinner.pack()
        htframe.pack()
        
        execute_button = Tkinter.Button(self,\
                                        command=lambda : self.execute.setValue( True ),\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)

    def ltfunc( self, *args ):
        s = self.lt_var.get()
        if not s=="":
            self.lowThreshold.setValue( float(s) )
            
    def htfunc( self, *args ):
        s = self.ht_var.get()
        if not s=="":
            self.highThreshold.setValue( float(s) )

            
# --- Compare segmentations ---
class CompareSegmentationsFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Compare segmentations"
        self["padx"] = 5
        self["pady"] = 5

        # H3DApi fields
        self.execute = SFBool()
        self.execute.setValue(False)
        self.infostring = SFString()
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.trueDelineation.updateMenu(labels)
        self.delineation.updateMenu(labels)
                        
    def createWidgets(self):
        self.trueDelineation = ImageMenu(self, text="True delineation")
        self.trueDelineation.pack()
        
        self.delineation = ImageMenu(self, text="Delineation")
        self.delineation.pack()
                
        execute_button = Tkinter.Button(self,\
                                        command=self.execute_func,\
                                        text="Execute")
        execute_button.config(width=10)
        execute_button.pack(padx=5, pady=5)
        
    def execute_func( self ):
        self.execute.setValue( True )
        tkMessageBox.showinfo("CompareSegmentations",\
                              self.infostring.getValue())
        
# --- Deformable simplex mesh ---
class MeshFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Deformable simplex mesh"
        self["padx"] = 5
        self["pady"] = 5

        self.initopendir = "../../data"
        self.initsavedir = "../../data"
        
        # H3DApi fields
        self.meshOpen = SFString()
        self.meshSave = SFString()
        self.resetButton = SFBool()
        self.refineButton = SFBool()
        self.activateButton = SFBool()
                
        self.meshRasterize = SFBool()
        self.gmWeight = SFFloat()
        self.potentialWeight = SFFloat()
        self.propagationWeight = SFFloat()
        self.propagationZeroLevel = SFFloat()
        self.vectorWeight = SFFloat()
        self.interactionWeight = SFFloat()
        self.interactionWeight.setValue(2.0)
        self.interactionSize = SFInt32()
        self.interactionSize.setValue(1)
        self.spatial = SFFloat()
        self.spatial.setValue(1.0)
        self.external = SFFloat()
        self.external.setValue(0.2)
        
        self.renderMode = SFInt32()
        self.scaling = SFFloat()
        self.scaling.setValue(1.0)
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.output.updateMenu(labels)
        self.gradmag.updateMenu(labels)
        self.potential.updateMenu(labels)
        self.propagation.updateMenu(labels)
        self.vector.updateMenu(labels)

    def render_var_func( self, name, index, mode ):
        v = self.render_var.get()
        if v=="Wire":
            self.renderMode.setValue(0)
        elif v=="Flat":
            self.renderMode.setValue(1)
        elif v=="Smooth":
            self.renderMode.setValue(2)
        
    def createWidgets(self):
        f = Tkinter.Frame(self)
        
        open_button = Tkinter.Button(f,\
                                     command=self.openfunc,\
                                     text="Open")
        open_button.pack(side="left", fill="x", expand=1)
        
        save_button = Tkinter.Button(f,\
                                     command=self.savefunc,\
                                     text="Save")
        save_button.pack(side="left", fill="x", expand=1)

        reset_button = Tkinter.Button(f,\
                                      command=lambda : self.resetButton.setValue( True ),\
                                      text="Create/Reset")
        reset_button.pack(side="left", fill="x", expand=1)

        refine_button = Tkinter.Button(f,\
                                       command=lambda : self.refineButton.setValue( True ),\
                                       text="Refine")
        refine_button.pack(side="left", fill="x", expand=1)

        rasterize_button = Tkinter.Button(f,\
                                          command=lambda : self.meshRasterize.setValue( True ),\
                                          text="Rasterize")
        rasterize_button.pack(side="right", fill="x", expand=1)
        f.grid(row=0,column=0,columnspan=3,sticky="ew")
        
        self.output = ImageMenu(self, text="Output")
        self.output.grid(row=1, column=0)
        
        rendermodeframe = Tkinter.Frame(self)
        Tkinter.Label(rendermodeframe, text="Render mode").pack(anchor="w")
        self.render_var = Tkinter.StringVar()
        self.render_var.set("Wire")
        self.render_var.trace_variable("w", self.render_var_func)        
        om = Tkinter.OptionMenu(rendermodeframe, self.render_var, "Wire", "Flat", "Smooth")
        om.config(width=10)
        om.pack()
        rendermodeframe.grid(row=1, column=1)

        tmpframe = Tkinter.Frame(self)
        
        helptext = Tkinter.Label(tmpframe, justify="left", bg="gray", text="Button 3: Translation\nButton 4: Interaction")
        helptext.pack(pady=5)

        self.activate_var = Tkinter.IntVar()
        self.activate_var.set( 0 )
        activate_button = Tkinter.Checkbutton(tmpframe,\
                                              text="Activate",\
                                              variable=self.activate_var,\
                                              command=lambda : self.activateButton.setValue(self.activate_var.get()) )
        activate_button.pack()
        
        tmpframe.grid(row=1,column=2,padx=10,pady=5)
        
        scaling_scale = Tkinter.Scale(self,\
                                      command=lambda x: self.scaling.setValue( float(x) ),\
                                      from_=0.01,\
                                      to=5.0,\
                                      resolution=0.01,\
                                      label="Scale",\
                                      orient="horizontal")
        scaling_scale.grid(row=2, column=2)
        scaling_scale.set(self.scaling.getValue())

        spatial_scale =  Tkinter.Scale(self,\
                                       command=lambda x: self.spatial.setValue( float(x) ),\
                                       from_=0.0,\
                                       to=1.0,\
                                       resolution=0.01,\
                                       label="Spatial",\
                                       orient="horizontal")
        spatial_scale.grid(row=3, column=2)
        spatial_scale.set(self.spatial.getValue())

        external_scale =  Tkinter.Scale(self,\
                                        command=lambda x: self.external.setValue( float(x) ),\
                                        from_=0.0,\
                                        to=1.0,\
                                        resolution=0.01,\
                                        label="External",\
                                        orient="horizontal")
        external_scale.grid(row=4, column=2)
        external_scale.set(self.external.getValue())
        
        interactionsz_scale = Tkinter.Scale(self,\
                                            command=lambda x: self.interactionSize.setValue( int(x) ),\
                                            from_=0,\
                                            to=5,\
                                            resolution=1,\
                                            label="Interaction size",\
                                            orient="horizontal")
        interactionsz_scale.grid(row=5, column=2)
        interactionsz_scale.set(self.interactionSize.getValue())
        
        interactionw_scale = Tkinter.Scale(self,\
                                           command=lambda x: self.interactionWeight.setValue( float(x) ),\
                                           from_=-10,\
                                           to=10,\
                                           resolution=0.01,\
                                           label="Interaction weight",\
                                           orient="horizontal")
        interactionw_scale.grid(row=6, column=2)
        interactionw_scale.set(self.interactionWeight.getValue())
        
        self.gradmag = ImageMenu(self, text="GradMag")
        self.gradmag.grid(row=2, column=0)
        
        gradmagw_scale = Tkinter.Scale(self,\
                                       command=lambda x: self.gmWeight.setValue( float(x) ),\
                                       from_=-10,\
                                       to=10,\
                                       resolution=0.01,\
                                       label="Weight",\
                                       orient="horizontal")
        gradmagw_scale.grid(row=2, column=1)
        gradmagw_scale.set( self.gmWeight.getValue() )
        
        self.potential = ImageMenu(self, text="Potential")
        self.potential.grid(row=3, column=0)

        potentialw_scale = Tkinter.Scale(self,\
                                         command=lambda x: self.potentialWeight.setValue( float(x) ),\
                                         from_=-10,\
                                         to=10,\
                                         resolution=0.01,\
                                         label="Weight",\
                                         orient="horizontal")
        potentialw_scale.grid(row=3, column=1)
        potentialw_scale.set( self.potentialWeight.getValue() )
        
        self.propagation = ImageMenu(self, text="Propagation")
        self.propagation.grid(row=4, column=0)

        propagationw_scale = Tkinter.Scale(self,\
                                           command=lambda x: self.propagationWeight.setValue( float(x) ),\
                                           from_=-10,\
                                           to=10,\
                                           resolution=0.01,\
                                           label="Weight",\
                                           orient="horizontal")
        propagationw_scale.grid(row=4, column=1)
        propagationw_scale.set( self.propagationWeight.getValue() )

        propagationzl_scale = Tkinter.Scale(self,\
                                            command=lambda x: self.propagationZeroLevel.setValue( float(x) ),\
                                            from_=0,\
                                            to=1,\
                                            resolution=0.01,\
                                            label="ZeroLevel",\
                                            orient="horizontal")
        propagationzl_scale.grid(row=5, column=1)
        propagationzl_scale.set( self.propagationZeroLevel.getValue() )
        
        self.vector = ImageMenu(self, text="Vector")
        self.vector.grid(row=6, column=0)

        vectorw_scale = Tkinter.Scale(self,\
                                      command=lambda x: self.vectorWeight.setValue( float(x) ),\
                                      from_=-10,\
                                      to=10,\
                                      resolution=0.01,\
                                      label="Weight",\
                                      orient="horizontal")
        vectorw_scale.grid(row=6, column=1)
        vectorw_scale.set( self.vectorWeight.getValue() )
        
    def openfunc( self ):
        fnm = tkFileDialog.askopenfilename(initialdir=self.initopendir)
        if not fnm=="":
            self.meshOpen.setValue( fnm )
            self.initopendir = os.path.split( fnm )[0]
            
    def savefunc( self ):
        fnm = tkFileDialog.asksaveasfilename(initialdir=self.initsavedir)
        if not fnm=="":
            self.meshSave.setValue( fnm )
            self.initsavedir = os.path.split( fnm )[0]



# --- Live marching ---
class LiveMarchingFrame(Tkinter.LabelFrame):
    def __init__(self, master=None):
        Tkinter.LabelFrame.__init__(self, master)
        self["text"] = "Live marching"
        self["padx"] = 5
        self["pady"] = 5
        
        # H3DApi fields
        self.clearPoints = SFBool()
        self.clearPoints.setValue( False )

        self.rasterizeButton = SFBool()
        self.rasterizeButton.setValue( False )
        
        self.active = SFBool()
        self.active.setValue( False )

        self.minBacktrackDistance = SFFloat()
        self.minBacktrackDistance.setValue( 5.0 )
        
        self.createWidgets()

    def refreshMenus(self, labels):
        self.image.updateMenu(labels)
        self.output.updateMenu(labels)
        
    def createWidgets(self):
        self.image = ImageMenu(self, text="Cost image")
        self.image.grid(row=0, column=0)

        self.output = ImageMenu(self, text="Output")
        self.output.grid(row=0, column=1)

        buttonframe = Tkinter.Frame(self)
        
        clear_button = Tkinter.Button(buttonframe,\
                                      command=lambda : self.clearPoints.setValue( True ),\
                                      text="Clear points")
        clear_button.pack(side="top", fill="both", expand=1)

        rasterize_button = Tkinter.Button(buttonframe,\
                                          command=lambda : self.rasterizeButton.setValue( True ),\
                                          text="Rasterize")
        rasterize_button.pack(side="top", fill="both", expand=1)

        buttonframe.grid(row=1, column=0, pady=5, sticky="nsew")

        mindist_scale = Tkinter.Scale(self,\
                                      command=lambda x: self.minBacktrackDistance.setValue( float(x) ),\
                                      from_=1,\
                                      to=100,\
                                      resolution=0.01,\
                                      label="Min backtrack dist",\
                                      orient="horizontal")
        mindist_scale.grid(row=1, column=1)
        mindist_scale.set( self.minBacktrackDistance.getValue() )

        rightframe = Tkinter.Frame(self)

        helptext = Tkinter.Label(rightframe, justify="left", bg="gray", text="Button 8: Track cursor\nButton 7: Add point\nButton 6: Delete point\nButton 5: Close and rasterize")
        helptext.grid(row=0, column=0, padx=10, pady=5)
        
        self.active_var = Tkinter.IntVar()
        self.active_var.set( 0 )
        active_button = Tkinter.Checkbutton(rightframe,\
                                            text="Active",\
                                            variable=self.active_var,\
                                            command=lambda : self.active.setValue(self.active_var.get()) )
        active_button.grid(row=1, column=0)

        rightframe.grid(row=0, column=2, rowspan=2)



root = Tkinter.Tk()
root.title( "wish" )

fileFrame = FileFrame(master=root)
fileFrame.pack(fill="both", padx=2, pady=2)

vol_et_frame = Tkinter.Frame(master=root)
volumeFrame = VolumeFrame(master=vol_et_frame)
volumeFrame.pack(side="left", fill="both", expand=1)
editTextureFrame = EditTextureFrame(master=vol_et_frame)
editTextureFrame.pack(side="left", fill="both", expand=1)
vol_et_frame.pack(fill="both", padx=2, pady=2)

hapticsFrame = HapticsFrame(master=root)
hapticsFrame.pack(fill="both", padx=2, pady=2)

# Filters
filterroot = Tkinter.Toplevel()
filterroot.title( "wish - filters" )

r = 0
c = 0
thresholdFrame = ThresholdFrame(master=filterroot)
thresholdFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

gsFrame = GSFrame(master=filterroot)
gsFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

ggmFrame = GGMFrame(master=filterroot)
ggmFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

ggFrame = GGFrame(master=filterroot)
ggFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

gvfFrame = GVFFrame(master=filterroot)
gvfFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

sigmoidFrame = SigmoidFrame(master=filterroot)
sigmoidFrame.grid(row=r, column=c, sticky="nsew")
r = r+1
c = 0

bilateralFrame = BilateralFrame(master=filterroot)
bilateralFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

fuzzyFrame = FuzzyFrame(master=filterroot)
fuzzyFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

fastMarchingFrame = FastMarchingFrame(master=filterroot)
fastMarchingFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

potentialFrame = PotentialFrame(master=filterroot)
potentialFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

distanceMapFrame = DistanceMapFrame(master=filterroot)
distanceMapFrame.grid(row=r, column=c, sticky="nsew")
c = c+1

compareFrame = CompareSegmentationsFrame(master=filterroot)
compareFrame.grid(row=r, column=c, sticky="nsew")

# Volume rendering
volrenroot = Tkinter.Toplevel()
volrenroot.title( "wish - volume rendering" )

volumeRendererFrame = VolumeRendererFrame(master=volrenroot)
volumeRendererFrame.pack(side="left", fill="both", expand=1, padx=2, pady=2)

# Deformable mesh
meshroot = Tkinter.Toplevel()
meshroot.title( "wish - segmentation" )

meshFrame = MeshFrame(master=meshroot)
meshFrame.pack(side="top", fill="both", expand=1, padx=2, pady=2)

liveMarchingFrame = LiveMarchingFrame(master=meshroot)
liveMarchingFrame.pack(side="top", fill="both", expand=1, padx=2, pady=2)

# Update Tkinter stuff
def refresh():
    volumeFrame.refresh()
    root.update()
    #root.lift()
    filterroot.update()
    volrenroot.update();
    meshroot.update()
    
def refreshMenus(labels):
    volumeFrame.refreshMenus(labels)
    hapticsFrame.refreshMenus(labels)
    editTextureFrame.refreshMenus(labels)
    thresholdFrame.refreshMenus(labels)
    gsFrame.refreshMenus(labels)
    ggmFrame.refreshMenus(labels)
    ggFrame.refreshMenus(labels)
    gvfFrame.refreshMenus(labels)
    sigmoidFrame.refreshMenus(labels)
    bilateralFrame.refreshMenus(labels)
    fastMarchingFrame.refreshMenus(labels)
    fuzzyFrame.refreshMenus(labels)
    potentialFrame.refreshMenus(labels)
    distanceMapFrame.refreshMenus(labels)
    compareFrame.refreshMenus(labels)
    volumeRendererFrame.refreshMenus(labels)
    meshFrame.refreshMenus(labels)
    liveMarchingFrame.refreshMenus(labels)
    
def traverseSG():
    try:
        refresh()
    except:
        pass
