///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

// the texture matrix
uniform mat4 textureMatrix;

// the volume textures
uniform sampler3D texture;
uniform sampler3D secondaryTexture;
uniform sampler3D overlay;

// colormap
uniform sampler2D colormap;

// render mode
uniform int renderMode;

// the window and level, here normalized
uniform float window;
uniform float level;

// the texture coordinate
varying vec3 texcoord;

// region labels
vec4 regionColor(float reg_id) {
  float lbl=0.0;
  if( reg_id > 0.0 )
    lbl = floor(mod(reg_id*255.0, 8.0)) + 1.0;
  
  vec4 reg_col;
  if( lbl == 0.0 )
    reg_col=vec4(1,1,1,1);
  else if( lbl == 1.0 )
    reg_col=vec4(1,0,0,1);
  else if( lbl == 2.0 )
    reg_col=vec4(1,.5,0,1);
  else if( lbl == 3.0 )
    reg_col=vec4(1,1,0,1);
  else if( lbl == 4.0 )
    reg_col=vec4(0,1,0,1);
  else if( lbl == 5.0 )
    reg_col=vec4(0,1,1,1);
  else if( lbl == 6.0 )
    reg_col=vec4(0,.5,1,1);
  else if( lbl == 7.0 )
    reg_col=vec4(0,0,1,1);
  else //if( lbl == 8 )
    reg_col=vec4(1,0,1,1);
  
  return reg_col;
    
}

float windowLevel(float arg) {
  float wl = clamp( (arg-level)/window + 0.5, 0.0, 1.0 ); 
  return wl;
}

vec4 windowLevel(vec4 arg, int cmp) {
  float temp = (arg[cmp]-level)/window + 0.5;
  vec4 ret;
  ret.rgb = (temp/max(arg[cmp], 1e-3))*arg.rgb;
  ret.a = max(0.8, temp);
  return ret;
}

void main() {
  
  // final color
  vec4 final_color;
  
  // volume value
  vec4 volval = texture3D(texture, texcoord);
  float wl=windowLevel(volval.r);
  
  if( renderMode==0 ) {
    final_color = vec4(wl,wl,wl,1.0); 
  }
  else if( renderMode==1 ) {
    float wl=windowLevel(volval.r);
    vec4 overlay_value = texture3D(overlay, texcoord);
    if( overlay_value.r > 0.0 ) {
      vec4 rc = regionColor(overlay_value.r);
      final_color = vec4(max(0.3,1.1*wl)*rc.rgb, 1.0);
    }
    else {
      final_color = vec4(wl, wl, wl, 1.0);
    }
  }
  else if( renderMode==2 ) {
    vec4 sectex_value = texture3D(secondaryTexture, texcoord);
    if( sectex_value.r > 0 ) {
      vec3 cmap = texture2D(colormap, vec2(sectex_value.r, 0.5)).rgb;      
      final_color = vec4(max(0.3,wl)*cmap,1.0);
    }
    else {
      final_color = vec4(wl, wl, wl, 1.0);
    }  
  }
  else if( renderMode==3 ) {
    vec4 val = texture3D(texture, texcoord);
    vec4 wlc = windowLevel(val, 3);
    final_color = vec4(wlc.rgb, 1.0);
  }
  else {
  }
  
  gl_FragColor = final_color;
  
}
