///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

// the volume texture
uniform sampler3D volumeTexture;

// the window and level, here normalized
uniform float window;
uniform float level;

// the intensity component to base the window/level computation on
// default is red comp.
uniform int intensityComponent;

float windowLevel(float arg) {
  return (arg-level)/window + 0.5; 
}

void main() {
  
  // get the color from texture
  vec4 texcol = texture3D(volumeTexture, gl_TexCoord[0].xyz);
  
  // intensity component
  float tci = texcol.r;
  if( intensityComponent == 3 )
    tci = texcol.a;
  
  // compute window/level value 
  float wl = windowLevel(tci);
  
  // multiply the texture color with the fraction in order 
  // to obtain a color with intensity wl
  if( tci > 1e-6 )
    gl_FragColor.rgb = (wl/tci)*texcol.rgb;
  else
    gl_FragColor.rgb = vec3(wl,wl,wl);
  
  // we set alpha to 1
  gl_FragColor.a = 1.0;
  
}
