///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_HH_
#define _WISH_H3D_HH_

#include "H3DTypes.h"
#include "wishLinalg.hh"

#if defined (WIN32) || defined (__WIN32__)

#include <windows.h>
#ifdef WISHH3D_EXPORTS
#define WISHH3D_API __declspec(dllexport)
#else
#define WISHH3D_API __declspec(dllimport)
#endif
#ifdef _MSC_VER
// disable dll-interface warnings for stl-exports 
#pragma warning( disable: 4251 )
#endif
#endif

#ifdef __linux
#define wishH3D_API
#endif

namespace wishH3D {

#define WISH_H3D_BUG() std::cerr << "Line " << __LINE__ << " in " << __FUNCTION__ << " in " << __FILE__ << std::endl;
#define WISH_H3D_ERROR(MESSAGE) fprintf(stderr, "ERROR: %s (line %d in %s)\n", MESSAGE, __LINE__, __FILE__)

#define WISH_H3D_CREATE_START() fprintf(stderr, "%s::%s() - Start\n", getTypeName().c_str(), getTypeName().c_str())
#define WISH_H3D_CREATE_STOP() fprintf(stderr, "%s::%s() - Stop\n", getTypeName().c_str(), getTypeName().c_str())

#define WISH_H3D_INIT_START() fprintf(stderr, "%s::initialize() - Start\n", getTypeName().c_str())
#define WISH_H3D_INIT_STOP() fprintf(stderr, "%s::initialize() - Stop\n", getTypeName().c_str())

  inline wish::Vec3d Vec3Cast(const H3D::Vec3f &v) {
    return wish::Vec3d(v.x, v.y, v.z);
  }
  
  inline H3D::Vec3f Vec3Cast(const wish::Vec3d &v) {
    return H3D::Vec3f(float(v.x), float(v.y), float(v.z));
  }

  inline H3D::Matrix3f Matrix3Cast(const wish::Matrix3f &m) {
    return H3D::Matrix3f(m(0,0), m(0,1), m(0,2),
			 m(1,0), m(1,1), m(1,2),
			 m(2,0), m(2,1), m(2,2));
  }
  
  inline wish::Matrix3f Matrix3Cast(const H3D::Matrix3f &m) {
    return wish::Matrix3f(m[0][0], m[0][1], m[0][2],
			  m[1][0], m[1][1], m[1][2],
			  m[2][2], m[2][2], m[2][2]);
  }

  inline H3D::Matrix4f Matrix4Cast(const wish::Matrix4f &m) {
    return H3D::Matrix4f(m(0,0), m(0,1), m(0,2), m(0,3),
			 m(1,0), m(1,1), m(1,2), m(1,3),
			 m(2,0), m(2,1), m(2,2), m(2,3),
			 m(3,0), m(3,1), m(3,2), m(3,3));
  }
  
  inline wish::Matrix4f Matrix4Cast(const H3D::Matrix4f &m) {
    return wish::Matrix4f(m[0][0], m[0][1], m[0][2], m[0][3],
			  m[1][0], m[1][1], m[1][2], m[1][3],
			  m[2][0], m[2][1], m[2][2], m[2][3],
			  m[3][0], m[3][1], m[3][2], m[3][3]);
  }
  
  inline string getEnv(const string &str) {
    string ret = getenv( str.c_str() );
    if( ret=="" ) {
      cerr << "Could not find environment variable " << str << endl;
      exit(1);
    }
    return ret + "/";
  }
  
  // Duplicates the input field and routes it to the output
  template<class T>
  T* copyAndRouteField(T *input) {
    T* f = new T;
    input->route( f );
    return f;
  }
  template<class T>
  T* copyAndRouteField(const auto_ptr<T> &input) {
    T* f = new T;
    input->route( f );
    return f;
  }
}

#endif
