///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DCompareSegmentations.hh"

#include "wishCompareSegmentations.hh"

namespace wishH3D {
  
  void CompareSegmentations::Compare::update() {
    value = "";
    bool val = static_cast<SFBool*>( event.ptr )->getValue();
    
    if( val ) {
      CompareSegmentations *cs = 
	static_cast<CompareSegmentations*>( getOwner() );
      
      // execute
      Volume *vol = cs->volume->getValue();
      wish::ImageObject *td = 
	vol->getImage( cs->trueDelineationIndex->getValue() );
      if( !td ) {
	return;
      }
      wish::ImageObject *d = 
	vol->getImage( cs->delineationIndex->getValue() );
      if( !d ) {
	return;
      }
      
      wish::CompareSegmentations comp;
      comp.setTrueDelineation( td );
      comp.setDelineation( d );
      comp.execute();
      
      value = 
	"Sensitivity:\t" + 
	wish::util::typeToString(comp.getSensitivity()) + "\n" +
	"Specificity:\t" + 
	wish::util::typeToString(comp.getSpecificity()) + "\n" +
	"Precision:\t" + 
	wish::util::typeToString(comp.getPrecision()) + "\n";
    }
  }
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  CompareSegmentations::database( "wishH3DCompareSegmentations", 
				  &(newInstance<CompareSegmentations>), 
				  typeid( CompareSegmentations ),
				  &X3DChildNode::database );
  
  namespace CompareSegmentationsInternals {
    FIELDDB_ELEMENT( CompareSegmentations, volume, INPUT_OUTPUT );
    FIELDDB_ELEMENT( CompareSegmentations, compare, INPUT_OUTPUT );
    FIELDDB_ELEMENT( CompareSegmentations, trueDelineationIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( CompareSegmentations, delineationIndex, INPUT_OUTPUT );
  }
  
  CompareSegmentations::CompareSegmentations() : 
    compare( new Compare ),
    trueDelineationIndex( new SFInt32 ),
    delineationIndex( new SFInt32 )
  {
    
    this->type_name = "CompareSegmentations";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // negative means not added to volume
    trueDelineationIndex->setValue( -1 );
    delineationIndex->setValue( -1 );
    
    // dependencies
    compare->setOwner( this );
    
    WISH_H3D_CREATE_STOP();
  }
  
}
