///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "Shape.h"
#include "Appearance.h"
#include "Material.h"
#include "SmoothSurface.h"
#include "H3DHapticsDevice.h"
#include "HLFeedbackShape.h"

#include "wishH3DDeformableSimplexMesh.hh"

namespace wishH3D {
  
  void DeformableSimplexMesh::WriteMesh::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFString::update();
    if( value != "" )
      dm->simplex->writeMesh( value );
  }
  
  void DeformableSimplexMesh::ReadMesh::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFString::update();
    if( value != "" )
      dm->simplex->readMesh( value );
  }
  
  void DeformableSimplexMesh::RasterizeMesh::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFBool::update();
    if( value ) {
      Volume *vol = dm->volume->getValue();
      
      wish::ImageObject *im0 = vol->getImage(0);
      if( !im0 )
	return;
      
      int w = im0->width();
      int h = im0->height();
      int d = im0->depth();
      wish::Vec3f spc = im0->getSpacing();
      
      wish::Image8* im = new wish::Image8(w,h,d,spc.x,spc.y,spc.z);
      dm->simplex->rasterizeMesh( im );
      im->setLabel( "SimplexRast" );
      
      int i = dm->outputIndex->getValue();
      bool ok = vol->setImage( i, im );
      
      if( !ok )
	vol->addImage( im );
    }
  }
  
  void DeformableSimplexMesh::Scaler::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    static bool firsttime = true;
    H3DFloat s = static_cast<SFFloat*>(routes_in[0])->getValue();
    if( firsttime ) {
      ref = s;
      firsttime = false;
    }
    wish::SimplexMesh *simplex = dm->simplex.get();
    H3DFloat scl = H3DMax( 1.0f+s-ref, 0.1f );
    simplex->transform( wish::scaling<double>(scl,scl,scl) );
    simplex->updateGeometry();
    ref = s;
    value = s;
  }
  
  void DeformableSimplexMesh::Resetter::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFBool::update();
    if( value ) {
      cerr << "DeformableSimplexMesh::Resetter::update()" << endl;	
      dm->resetMesh();
    }
  }
  
  void DeformableSimplexMesh::Refiner::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFBool::update();
    if( value ) {
      cerr << "DeformableSimplexMesh::Refiner::update()" << endl;	
      dm->refineMesh();
    }
  }
  
  void DeformableSimplexMesh::Translator::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    Vec3f t = static_cast<SFVec3f*>(routes_in[0])->getValue();
    bool button = static_cast<SFBool*>(routes_in[1])->getValue();
    
    if( button ) {
      wish::SimplexMesh *simplex = dm->simplex.get();
      wish::Vec3d T = Vec3Cast( t-ref );
      simplex->transform(wish::translation<double>(T));
      simplex->updateGeometry();
    }
    
    ref = t;
    value = t;
  }
  
  void DeformableSimplexMesh::SpatialWeight::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFFloat::update();
    dm->simplex->setSpatialWeight(value);
  }
  
  void DeformableSimplexMesh::ExternalWeight::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFFloat::update();
    dm->simplex->setExternalWeight(value);
  }
  
  void DeformableSimplexMesh::ExternalForceWeight::update() {
    DeformableSimplexMesh *dm = 
      static_cast<DeformableSimplexMesh*>(getOwner());
    SFFloat::update();
    ef->setWeight(value);
  }      
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  DeformableSimplexMesh::database( "wishH3DDeformableSimplexMesh", 
				   &(newInstance<DeformableSimplexMesh>), 
				   typeid( DeformableSimplexMesh ),
				   &Group::database );
  
  namespace DeformableSimplexMeshInternals {
    FIELDDB_ELEMENT( DeformableSimplexMesh, volume, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, activateButton, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, interactionButton, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, renderMode, INPUT_OUTPUT );    
    FIELDDB_ELEMENT( DeformableSimplexMesh, resetButton, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, refineButton, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, scaling, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, translation, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, translationButton, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, writeMesh, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, readMesh, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, rasterizeMesh, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, imageIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, outputIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, gradientMagnitudeIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, potentialIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, propagationIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, vectorIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, spatialWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, externalWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, interactionWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, interactionSize, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, gradientMagnitudeWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, propagationWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, propagationZeroLevel, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, potentialWeight, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DeformableSimplexMesh, vectorWeight, INPUT_OUTPUT );
  }
  
  DeformableSimplexMesh::DeformableSimplexMesh() :
    simplex( new wish::SimplexMesh ),
    interactionButton( new SFBool ),
    activateButton( new SFBool ),
    resetButton( new SFBool ),
    resetter( new Resetter ),
    refineButton( new SFBool ),
    refiner( new Refiner ),
    renderMode( new SFInt32 ),
    scaling( new SFFloat ),
    translation( new SFVec3f ),
    translationButton( new SFBool ),
    scaler( new Scaler ),
    translator( new Translator ),
    writeMesh( new WriteMesh ),
    readMesh( new ReadMesh ),
    rasterizeMesh( new RasterizeMesh ),
    imageIndex( new SFInt32 ),
    outputIndex( new SFInt32 ),
    gradientMagnitudeIndex( new SFInt32 ),
    propagationIndex( new SFInt32 ),
    potentialIndex( new SFInt32 ),
    vectorIndex( new SFInt32 ),
    interactionForce( new wish::InteractionForce ),
    gradientMagnitudeForce( new wish::GradientMagnitudeForce ),
    potentialFieldForce( new wish::PotentialFieldForce ),
    propagationForce( new wish::PropagationForce ),
    vectorFieldForce( new wish::VectorFieldForce ),
    spatialWeight( new SpatialWeight ),
    externalWeight( new ExternalWeight ),
    interactionWeight( new ExternalForceWeight ),
    interactionSize( new SFInt32 ),
    gradientMagnitudeWeight( new ExternalForceWeight ),
    propagationWeight( new ExternalForceWeight ),
    propagationZeroLevel( new SFFloat ),
    potentialWeight( new ExternalForceWeight ),
    vectorWeight( new ExternalForceWeight ),
    forceEffect( new ForceEffect )
  {

    type_name = "DeformableSimplexMesh";
    database.initFields(this);

    WISH_H3D_CREATE_START();
    
    // dependencies
    writeMesh->setOwner( this );
    readMesh->setOwner( this );
    rasterizeMesh->setOwner( this );
    translator->setOwner( this );
    scaler->setOwner( this );
    resetter->setOwner( this );
    refiner->setOwner( this );
    
    translation -> routeNoEvent( translator );
    translationButton -> route( translator ); 
    scaling -> routeNoEvent( scaler );
    resetButton->route( resetter );
    refineButton->route( refiner );
    
    // render mode
    renderMode->setValue(0);
    
    // default button
    activateButton -> setValue(false);
    interactionButton -> setValue(false);
    translationButton -> setValue( false );
    
    // set the dependencies
    spatialWeight->setOwner(this);
    externalWeight->setOwner(this);
    
    interactionWeight->ef = interactionForce.get();
    gradientMagnitudeWeight->ef = gradientMagnitudeForce.get();
    potentialWeight->ef = potentialFieldForce.get();
    vectorWeight->ef = vectorFieldForce.get();
    propagationWeight->ef = propagationForce.get();
    
    // defaults
    interactionWeight->setValue( 0.0f );
    interactionSize->setValue( 1 );
    gradientMagnitudeWeight->setValue( 0.0f );
    potentialWeight->setValue( 0.0f );
    vectorWeight->setValue( 0.0f );
    propagationWeight->setValue( 0.0f );
    propagationZeroLevel->setValue( 0.8f );
    
    imageIndex->setValue(-1);
    outputIndex->setValue(-1);
    gradientMagnitudeIndex->setValue(-1);
    propagationIndex->setValue(-1);
    potentialIndex->setValue(-1);
    vectorIndex->setValue(-1);
    
    // routes
    renderMode->route( displayList );
    
    WISH_H3D_CREATE_STOP();
  };
  
  void DeformableSimplexMesh::initialize() {
   
    if( isInitialized() )
      return;
    
    WISH_H3D_INIT_START();
    
    // get the volume
    Volume *vol = volume -> getValue();
        
    // add external force data
    simplex -> addExternalForce( interactionForce.get() );
    simplex -> addExternalForce( gradientMagnitudeForce.get() );
    simplex -> addExternalForce( potentialFieldForce.get() );
    simplex -> addExternalForce( propagationForce.get() );     
    simplex -> addExternalForce( vectorFieldForce.get() );    
    
    WISH_H3D_INIT_STOP();
  }
  
  wish::SimplexVertex* 
  DeformableSimplexMesh::selectVertices(const Vec3f &pos, 
					const int &nbsize) {
    // unselect all vertices
    deselectVertices();
    
    // find the closest point
    wish::SimplexMesh::SimplexVertexIterator itv = simplex->vbegin();
    if( itv==simplex->vend() )
      return 0;
    
    Vec3f vpos = Vec3Cast( (*itv)->getPosition() );
    H3DFloat mindist = H3DFloat( (vpos-pos)*(vpos-pos) );
    wish::SimplexVertex *closest = *itv;
    itv++;
    for(; itv != simplex->vend(); itv++) {
      vpos = Vec3Cast( (*itv)->getPosition() );
      H3DFloat tmpdist = H3DFloat( (vpos-pos)*(vpos-pos) );
      if( tmpdist < mindist ) {
	mindist = tmpdist;
	closest = *itv;
      }
    }
    
    // iterate over vertex neighborhood and select vertices
    wish::SimplexVertex::NeighborhoodIterator nbit = closest->nbegin(nbsize);
    nbit.vertex->setSelected(true);
    nbit.current()->setSelected(true);
    while( nbit.next() ) {
      nbit.current()->setSelected(true);
    }
    
    // return the closest point
    return closest;
  }
  
  void DeformableSimplexMesh::deselectVertices() {
    simplex->selectVertices(false);
  }
  
  wish::SimplexFace* DeformableSimplexMesh::selectFaces(const Vec3f &pos, 
							const int &nbsize) {
    // unselect all vertices
    deselectFaces();      
    
    // find the closest point
    wish::SimplexMesh::SimplexFaceIterator itf = simplex->fbegin();
    if( itf==simplex->fend() )
      return 0;
    
    Vec3f fpos = Vec3Cast( (*itf)->getCenter() );
    H3DFloat mindist = H3DFloat( (fpos-pos)*(fpos-pos) );
    wish::SimplexFace *closest = *itf;
    itf++;
    for(; itf != simplex->fend(); itf++) {
      fpos = Vec3Cast( (*itf)->getCenter() );
      H3DFloat tmpdist = H3DFloat( (fpos-pos)*(fpos-pos) );
      if( tmpdist < mindist ) {
	mindist = tmpdist;
	closest = *itf;
      }
    }
    
    // iterate over face neighborhood and select faces
    wish::SimplexFace::NeighborhoodIterator nbit = closest->nbegin(nbsize);
    nbit.face->setSelected(true);
    nbit.current()->setSelected(true);
    
    while( nbit.next() ) {
      nbit.current()->setSelected(true);
      // also select the face vertices
      wish::SimplexFace::VertexIterator itv = nbit.current()->vbegin();
      itv.current()->setSelected(true);
      while( itv.next() ) {
	itv.current()->setSelected(true);
      }
    }
    
    // return the closest point
    return closest;
  }
  
  void DeformableSimplexMesh::deselectFaces() {
    simplex->selectFaces(false);
    simplex->selectVertices(false);
  }
  
  void DeformableSimplexMesh::updateExternalForces() {
    Volume *vol = volume->getValue();
    
    if( !vol )
      return;
    
    wish::ImageObject *im;
    
    // set to zero
    gradientMagnitudeForce->setImage(0);
    potentialFieldForce->setImage(0);
    propagationForce->setImage(0);
    vectorFieldForce->setImage(0);
    
    // gradient magnitude
    if( im = vol->getImage(gradientMagnitudeIndex->getValue()) ) {
      gradientMagnitudeForce->setImage( im );
    }
    
    // potential
    if( im = vol->getImage(potentialIndex->getValue()) ) {
      potentialFieldForce->setImage( im );
    }
    
    // propagation
    if( im = vol->getImage(propagationIndex->getValue()) ) {
      propagationForce->setImage( im );
      propagationForce->setZeroLevel( propagationZeroLevel->getValue() );
    }
    
    // vector
    if( im = vol->getImage(vectorIndex->getValue()) ) {
      vectorFieldForce->setImage( im );
    }
  }
  
  void DeformableSimplexMesh::render() {
    simplex->renderGL( renderMode->getValue() );
  }
  
  void DeformableSimplexMesh::traverseSG( TraverseInfo &ti ) {
    
    // base class
    Group::traverseSG(ti);
    
    // ugly...
    renderMode->touch();
      
    // get the volume
    Volume *vol = volume -> getValue();
    
    // get the probe position
    Vec3f xtip = vol -> probePosition -> getValue();
    
    //FORCE EFFECT STUFF
    // device to local matrix
    Matrix4f D2L = 
      ti.getAccInverseMatrix()*
      ti.getHapticsDevice(0)->positionCalibration->getValue();
    
    // rotation and scale part
    Matrix3f D2L_rs = D2L.getScaleRotationPart();
    
    // find scale factor (OBS! only uniform scaling allowed)
    float scl = 1.0e-3f / ( D2L_rs*Vec3f(1,0,0) ).length();
    
    // set force effect params
    forceEffect->transform = ti.getAccForwardMatrix();
    forceEffect->setStiffness( 100.0f );
    forceEffect->setDeviceScale( scl );
    
    static int count=0;
    //END FORCE EFFECT STUFF
    
    // interaction button pressed
    if( interactionButton->getValue() ) {
      // get the mean collected forces
      Vec3f mcf = Vec3Cast( interactionForce->getMeanCollectedForces() );
      
      // set force
      forceEffect->setForce( -mcf );
      
      // add force
      ti.addForceEffectToAll( forceEffect.get() );    
      
      // select faces
      selectFaces( xtip, interactionSize->getValue() );
      
      // reset
      interactionForce->resetCollectedForces();
      interactionForce->setCollectForces( true );
      
      // set the interaction position
      interactionForce->setWeight( interactionWeight->getValue() );
      interactionForce->setInteractionPosition( Vec3Cast(xtip) );
    }
    
    // interaction button released
    else {
      deselectFaces();
      interactionForce->setWeight(0.0);
    } 
    
    // activate button pressed
    if( activateButton->getValue() ) {
      updateExternalForces();
      simplex->deform();
      simplex->updateGeometry();
    }
    
  }
  
  HapticForceEffect::EffectOutput 
  DeformableSimplexMesh::ForceEffect::
  calculateForces( const HapticForceEffect::EffectInput &input ) {
    
    // rotational part of acc_forward_transform
    Matrix3f R( transform.getRotationPart() );
    
    // compute force
    this->computeForceFeedback();
    Vec3f f = this->getForce();
    
    // safety
    H3DFloat l = f.length();
    if( l > MAX_FORCE ) {
      cerr << "FORCE MAGNITUDE: " << l << endl;
      f = Vec3f(0,0,0);
    }
    
    // return force in world coordinates
    return EffectOutput( R*f );
  }
  
}  


  
