///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DDistanceMapFilter.hh"

#include "wishDistanceMapFilter.hh"

namespace wishH3D {
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  DistanceMapFilter::database( "wishH3DDistanceMapFilter", 
			       &(newInstance<DistanceMapFilter>), 
			       typeid( DistanceMapFilter ),
			       &FilterObject::database );
  
  namespace DistanceMapFilterInternals {
    FIELDDB_ELEMENT( DistanceMapFilter, inputIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( DistanceMapFilter, fromBackground, INPUT_OUTPUT );
    
    template<class InputImageType, class OutputImageType>
    void executeFilter(InputImageType *in,
		       OutputImageType* &out,
		       bool from_bg) {
      
      wish::DistanceMapFilter<InputImageType, OutputImageType> dmf;
      dmf.setInput( in );
      dmf.setInternal( from_bg );
      cerr << "Executing DistanceMapFilter..." << endl; 
      dmf.execute();
      cerr << "Execution time = " << dmf.getProcessTime() << " s" << endl;
      
      out = dmf.releaseOutput();
    }
    
  }
  
  DistanceMapFilter::DistanceMapFilter() :
    inputIndex( new SFInt32 ),
    fromBackground( new SFBool )
  {  
    this->type_name = "DistanceMapFilter";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // defaults
    inputIndex->setValue( -1 );
    fromBackground->setValue( true );

    WISH_H3D_CREATE_STOP();
    
  }
  
  wish::ImageObject* DistanceMapFilter::execute() {
    Volume *vol = volume->getValue();
    if( !vol ) 
      return 0;
    
    wish::ImageObject *input = vol->getImage( inputIndex->getValue() );
    if( !input )
      return 0;
    
    string dt = input->getDataType();
    if( dt=="uint8" ) {
      wish::Image8 *in = dynamic_cast<wish::Image8*>(input);
      wish::Image16 *out;
      DistanceMapFilterInternals::executeFilter( in, out,
						 fromBackground->getValue() );
      return out;
    }
    else if( dt=="uint16" ) {
      wish::Image16 *in = dynamic_cast<wish::Image16*>(input);
      wish::Image16 *out;
      DistanceMapFilterInternals::executeFilter( in, out,
						 fromBackground->getValue() );
      return out;
    }
    else {
      return 0;
    }
    
  }
  
}
