///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DEditVolumeTexture.hh"

namespace wishH3D {
  
  void EditVolumeTexture::UpdateTextureField::update() {
    static_cast<EditVolumeTexture*>(getOwner())->updateTexture();
  }
  
  void EditVolumeTexture::ClearAll::update() {
    SFBool::update();
    if( value ) {
      EditVolumeTexture *ev = static_cast<EditVolumeTexture*>(getOwner());
      Volume *vol = ev->volume->getValue();
      int ind = ev->imageIndex->getValue();
      wish::Image8 *im = dynamic_cast<wish::Image8*>(vol->getImage(ind));
      if( im ) {
	im->reset(0);
      }
      ev->updateTexture();
    }
  }
  
  void EditVolumeTexture::ClearLabel::update() {
    SFBool::update();
    if( value ) {
      EditVolumeTexture *ev = static_cast<EditVolumeTexture*>(getOwner());
      Volume *vol = ev->volume->getValue();
      int ind = ev->imageIndex->getValue();
      wish::Image8 *im = dynamic_cast<wish::Image8*>(vol->getImage(ind));
      if( im ) {
	int lbl = ev->editLabel->getValue();
	for(int k=0;k<im->totalSize();k++) {
	  if( (*im)[k]==lbl ) (*im)[k] = 0;
	}
	ev->updateTexture();
      }
    }
  }
  
  void EditVolumeTexture::EditField::update() {
    Vec3f v = static_cast<SFVec3f*>(routes_in[0])->getValue();
    bool add = static_cast<SFBool*>(routes_in[1])->getValue();
    bool remove = static_cast<SFBool*>(routes_in[2])->getValue();
    EditVolumeTexture *ev = static_cast<EditVolumeTexture*>( getOwner() );
    
    if( add )
      ev->drawSphere();
    
    if( remove )
      ev->eraseSphere();
    
    value = v;
  }
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase EditVolumeTexture::database( "wishH3DEditVolumeTexture", 
					       &(newInstance<EditVolumeTexture>), 
					       typeid( EditVolumeTexture ),
					       &VolumeTexture::database );
  
  namespace EditVolumeTextureInternals {
    FIELDDB_ELEMENT( EditVolumeTexture, editSize, INPUT_OUTPUT );    
    FIELDDB_ELEMENT( EditVolumeTexture, editPosition, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, editAdd, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, editRemove, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, editLabel, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, clearAll, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, clearLabel, INPUT_OUTPUT );
    FIELDDB_ELEMENT( EditVolumeTexture, updateTextureField, INPUT_OUTPUT );
  }
  
  EditVolumeTexture::EditVolumeTexture() :
    editSize( new SFFloat ),
    editPosition( new SFVec3f ),
    editAdd( new SFBool ),
    editRemove( new SFBool ),
    editLabel( new SFInt32 ),
    clearAll( new ClearAll ),    
    clearLabel( new ClearLabel ),
    updateTextureField( new UpdateTextureField ),
    editField( new EditField )
  {
    
    type_name = "EditVolumeTexture";
    database.initFields(this);

    WISH_H3D_CREATE_START();
    
    // no interpolation
    this->interpolate_string = "NEAREST_PIXEL";
    
    // dependencies
    clearAll->setOwner( this );
    clearLabel->setOwner( this );
    editField->setOwner( this );
    updateTextureField->setOwner( this );

    // defaults
    editSize->setValue(3.0f);
    editAdd->setValue(false);
    editRemove->setValue(false);
    editLabel->setValue( 255 );
    clearAll->setValue( false );
    clearLabel->setValue( false );
    
    // routes
    editPosition->route(editField);
    editAdd->route(editField);
    editRemove->route(editField);
    
    WISH_H3D_CREATE_STOP();
  }
  
  void EditVolumeTexture::updateTexture() {
    //this->generateTexture();
    this->image->touch();
  }
  
  void EditVolumeTexture::drawSphere() {
    Volume *vol = volume->getValue();
    int ind = imageIndex->getValue();
    wish::Image8 *im = dynamic_cast<wish::Image8*>(vol->getImage(ind));
    if( im ) {
      int lbl = editLabel->getValue();
      int r = int( floor( editSize->getValue() ) );
      int rq = r*r;
      Vec3f p = editPosition->getValue();
      int x = int(floor(p.x));
      int y = int(floor(p.y));
      int z = int(floor(p.z));
      for(int i=-r; i<=r; i++) {
	for(int j=-r; j<=r; j++) {
	  for(int k=-r; k<=r; k++) {
	    if( (i*i+j*j+k*k) < rq ) {
	      im->setSafe(lbl,x+k,y+j,z+i);
	    }
	  }
	}
      }
      updateTexture();
    }
  
  }
  
  void EditVolumeTexture::eraseSphere() {
    Volume *vol = volume->getValue();
    int ind = imageIndex->getValue();
    wish::Image8 *im = dynamic_cast<wish::Image8*>(vol->getImage(ind));
    if( im ) {
      int lbl = editLabel->getValue();
      int r = int( floor( editSize->getValue() ) );
      int rq = r*r;
      Vec3f p = editPosition->getValue();
      int x = int(floor(p.x));
      int y = int(floor(p.y));
      int z = int(floor(p.z));
      for(int i=-r; i<=r; i++) {
	for(int j=-r; j<=r; j++) {
	  for(int k=-r; k<=r; k++) {
	    if( (i*i+j*j+k*k) < rq ) {
	      im->setSafe(0,x+k,y+j,z+i);
	    }
	  }
	}
      }
      updateTexture();
    }
    
  }
  
}
