///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_EDIT_VOLUME_TEXTURE_HH_
#define _WISH_H3D_EDIT_VOLUME_TEXTURE_HH_

#include "X3DChildNode.h"
#include "wishH3DVolumeChild.hh"
#include "wishH3DVolumeTexture.hh"

namespace wishH3D {
  
  class WISHH3D_API EditVolumeTexture : public VolumeTexture {
  public:
    typedef VolumeTexture BaseClass;
    
    // the database for this node
    static H3DNodeDatabase database;
    
    // field that edits the volume when it updates
    // routes_in[0] = editAdd
    // routes_in[1] = editRemove
    class EditField : 
      public TypedField< PeriodicUpdate<SFVec3f>, 
      Types<SFVec3f, SFBool, SFBool> > {
      virtual void update();
    };
    
    // field that just updates the texture, route anything from 
    // outside into this field in order to update the texture
    class UpdateTextureField : public AutoUpdate<Field> {
      virtual void update();
    };

    // field that clears the image
    class ClearAll : public AutoUpdate<SFBool> {
      virtual void update();
    };
    
    // field that clears the image specific label
    class ClearLabel : public AutoUpdate<SFBool> {
      virtual void update();
    };
    
    // constructor
    EditVolumeTexture();
    
    // update texture
    void updateTexture();
    
    // size of tool
    auto_ptr<SFFloat> editSize;
    
    // position of tool
    auto_ptr<SFVec3f> editPosition;    
    
    // add
    auto_ptr<SFBool> editAdd;
    
    // remove
    auto_ptr<SFBool> editRemove;
    
    // the label
    auto_ptr<SFInt32> editLabel;
    
    // clear all
    auto_ptr<ClearAll> clearAll;
    
    // clear label
    auto_ptr<ClearLabel> clearLabel;
    
    // update texture
    auto_ptr<UpdateTextureField> updateTextureField;
    
  protected:
    // the edit field
    auto_ptr<EditField> editField;
    
    // editing functions
    void drawSphere();
    void eraseSphere();
  };
  
}

#endif
