///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DFilterObject.hh"

namespace wishH3D {
  
  void FilterObject::ExecuteFilter::update() {
    SFBool::update();
    if( value ) {
      FilterObject *fo = static_cast<FilterObject*>( getOwner() );
      
      // execute filter
      wish::ImageObject *out = fo->execute();
      
      if( !out )
	return;
      
      // add output to volume
      Volume *vol = fo->volume->getValue();
      if( !vol ) {
	delete out;	
	return;
      }
      
      int i = fo->outputIndex->getValue();
      bool ok = vol->setImage( i, out );
      
      if( !ok )
	vol->addImage( out );
    }
  }
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  FilterObject::database( "wishH3DFilterObject", 
			  &(newInstance<FilterObject>), 
			  typeid( FilterObject ),
			  &X3DChildNode::database );
  
  namespace FilterObjectInternals {
    FIELDDB_ELEMENT( FilterObject, volume, INPUT_OUTPUT );
    FIELDDB_ELEMENT( FilterObject, executeFilter, INPUT_OUTPUT );
    FIELDDB_ELEMENT( FilterObject, outputIndex, INPUT_OUTPUT );
  }
  
  FilterObject::FilterObject() : 
    executeFilter( new ExecuteFilter ),
    outputIndex( new SFInt32 )
  {
    
    this->type_name = "FilterObject";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // negative means not added to volume
    outputIndex->setValue( -1 );
    
    // dependencies
    executeFilter->setOwner( this );
    
    // default values
    executeFilter->setValue( false );
    
    WISH_H3D_CREATE_STOP();
    
  }
  

  
}
