///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DGaussianSmoothingFilter.hh"

#include "wishGaussianSmoothingFilter.hh"

namespace wishH3D {
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  GaussianSmoothingFilter::database( "wishH3DGaussianSmoothingFilter", 
				     &(newInstance<GaussianSmoothingFilter>), 
				     typeid( GaussianSmoothingFilter ),
				     &FilterObject::database );
  
  namespace GaussianSmoothingFilterInternals {
    FIELDDB_ELEMENT( GaussianSmoothingFilter, inputIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( GaussianSmoothingFilter, sigma, INPUT_OUTPUT );
    
    template<class InputImageType, class OutputImageType>
    void executeFilter(InputImageType *in,
		       OutputImageType* &out,
		       H3DFloat sigma) {
      
      wish::GaussianSmoothingFilter<InputImageType, OutputImageType> gsf;
      gsf.setInput( in );
      gsf.setSigma(sigma);
      cerr << "Executing GaussianSmoothingFilter..." << endl; 
      gsf.execute();
      cerr << "Execution time = " << gsf.getProcessTime() << " s" << endl;
      
      out = gsf.releaseOutput();
    }
    
  }
  
  GaussianSmoothingFilter::GaussianSmoothingFilter() :
    inputIndex( new SFInt32 ),
    sigma( new SFFloat )
  {  
    this->type_name = "GaussianSmoothingFilter";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // defaults
    inputIndex->setValue( -1 );
    sigma->setValue(1.0f);
    
    WISH_H3D_CREATE_STOP();
    
  }
  
  wish::ImageObject* GaussianSmoothingFilter::execute() {
    Volume *vol = volume->getValue();
    if( !vol ) 
      return 0;
    
    wish::ImageObject *input = vol->getImage( inputIndex->getValue() );
    if( !input )
      return 0;
    
    string dt = input->getDataType();
    if( dt=="uint8" ) {
      wish::Image8 *in = dynamic_cast<wish::Image8*>(input);
      wish::Image8 *out;
      GaussianSmoothingFilterInternals::
	executeFilter( in, out, 
		       sigma->getValue() );
      return out;
    }
    else if( dt=="uint16" ) {
      wish::Image16 *in = dynamic_cast<wish::Image16*>(input);
      wish::Image16 *out;
      GaussianSmoothingFilterInternals::
	executeFilter( in, out, sigma->getValue() );
      return out;
    }
	else {
		return 0;
	}
    
  }
  
}
