///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_LIVE_MARCHING_HH_
#define _WISH_H3D_LIVE_MARCHING_HH_

#include "Group.h"
#include "X3D.h"
#include "Threads.h"
#include "TimeStamp.h"

#include "wishH3DVolume.hh"
#include "wishH3DVolumeChild.hh"

#include "wishLiveMarching.hh"

namespace wishH3D {
  
  /**
     wishH3DLiveMarching
  */
  class WISHH3D_API LiveMarching : public Group, public VolumeChild {
    
  public:
    //! X3D interface
    static H3DNodeDatabase database;
    
    //! add point    
    struct AddPoint : public AutoUpdate<SFBool> {
      virtual void update();
    };
    friend struct AddPoint;
    
    //! delete point    
    struct DeletePoint : public AutoUpdate<SFBool> {
      virtual void update();
    };
    friend struct DeletePoint;
    
    //! close contour 
    struct CloseContour : public AutoUpdate<SFBool> {
      virtual void update();
    };
    friend struct CloseContour;
    
    //! reset
    struct ClearPoints : public AutoUpdate<SFBool> {
      virtual void update();
    };
    friend struct ClearPoints;
    
    //! change image slice from input image and slice translation
    struct ImageSlice : 
      public AutoUpdate< TypedField< Field, SFInt32 > > {
      virtual void update();      
    };
    friend struct ImageSlice;
    
    //! rasterize current contour to output image
    struct Rasterizer : public AutoUpdate<SFBool> {
      virtual void update();      
    };
    friend struct Rasterizer;

    //! constructor
    LiveMarching();
    
    //! initialize function
    virtual void initialize();
    
    //! traverse SG (activate the model)
    virtual void traverseSG( TraverseInfo &ti );
    
    //! render
    virtual void render();
    
    //! the cost image
    auto_ptr<SFInt32> imageIndex;
    
    //! the output image
    auto_ptr<SFInt32> outputIndex;
    
    //! the current slice translation
    auto_ptr<SFVec3f> sliceTranslation;
    
    //! active
    auto_ptr<SFBool> active;

    //! minimum backtrack distance
    // if the distance between cursor and seed is smaller than this value
    // no backtracking will be performed, the result will be a straight 
    // line segment
    auto_ptr<SFFloat> minBacktrackDistance;
    
    //! updates the image slice from the input image and the slice translation
    auto_ptr<ImageSlice> imageSlice;
    
    //! buttons
    auto_ptr<SFBool> addButton;
    auto_ptr<AddPoint> addPoint;
    
    auto_ptr<SFBool> deleteButton;
    auto_ptr<DeletePoint> deletePoint;
    
    auto_ptr<SFBool> closeButton;
    auto_ptr<CloseContour> closeContour;
    
    auto_ptr<SFBool> clearButton;
    auto_ptr<ClearPoints> clearPoints;
    
    auto_ptr<SFBool> interactionButton;
    
    auto_ptr<SFBool> rasterizeButton;
    auto_ptr<Rasterizer> rasterizer;
        
    //! the live marching object
    auto_ptr<wish::LiveMarching> live_marching;
    
  protected:
    wish::ImageObject *cost2D;
    int current_slice;
    
    void setCostSlice(wish::ImageObject *cs) { 
      delete cost2D;
      cost2D = cs;
      live_marching->setInput( cost2D );
      live_marching->initialize();
    }
    
    wish::ImageObject* getCostSlice() const {
      return cost2D;
    }
    
  };
}
#endif
