///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DSFTFTexture.hh"

#include "TextureProperties.h"

using namespace H3D;

namespace wishH3D {
  
  // nr elements i the 1D texture 
  const int SFTFTexture::NTEX = 512;
  
  SFTFTexture::SFTFTexture() {
    
    std::cerr << "SFTFTexture::SFTFTexture() - Start!" << std::endl;
    
    PixelTexture *tex = new PixelTexture; 
        
    TextureProperties *tp = new TextureProperties;
    tp->boundaryModeS->setValue("CLAMP_TO_EDGE");
    tp->boundaryModeT->setValue("CLAMP_TO_EDGE");
    tp->boundaryModeR->setValue("CLAMP_TO_EDGE");
    tp->minificationFilter->setValue("AVG_PIXEL");
    tp->magnificationFilter->setValue("AVG_PIXEL");
    tp->textureCompression->setValue("DEFAULT");
    tex->textureProperties->setValue(tp);
    
    unsigned int data[NTEX];
    memset(data, 0, sizeof(int)*NTEX);
    tex->image->setValue( new PixelImage(NTEX,1,1,
					 8*4,
					 Image::RGBA,
					 Image::UNSIGNED,
					 (unsigned char*) data, true) );
    value = tex;
    
    std::cerr << "SFTFTexture::SFTFTexture() - Stop!" << std::endl;
  }
  
  // a maximum of four transfer functions can be routed in to us
  void SFTFTexture::update() {
    
    typedef TransferFunction* TransferFunctionPointer;
    
    // nr of routes
    int rsize = int( routes_in.size() );
    
    // pointers to tf:s
    TransferFunctionPointer tf[4];
    tf[0] = tf[1] = tf[2] = tf[3] = 0;
    for( int i=0; i<rsize; i++ ) {
      tf[i] = static_cast<SFTransferFunction*>(routes_in[i])->getValue();
    }
    
    // image
    PixelTexture *tex = static_cast<PixelTexture*>( value.get() );
    PixelImage *im = static_cast<PixelImage*>( tex->image->getValue() );
    
    // loop image and evaluate tf:s
    unsigned int *ptr = (unsigned int*)(im->getImageData());
    for(int i=0; i<NTEX; i++) {
      
      unsigned char *p = (unsigned char*)(&ptr[i]);      
      float t = float(i)/(NTEX-1);
      float val;
      
      for( int j=0; j<rsize; j++ ) {
	val = tf[j]->evaluate(t);
	if( val < 0.0 ) val = 0.0;
	if( val > 1.0 ) val = 1.0;
	p[j] = (unsigned char) ( 255.0f*val );
      }
      
    }
    
    // make things update
    tex->image->setValue( im );
    
  }
  
}
