///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DSigmoidFilter.hh"

#include "wishSigmoidFilter.hh"

namespace wishH3D {
  
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  SigmoidFilter::database( "wishH3DSigmoidFilter", 
			   &(newInstance<SigmoidFilter>), 
			   typeid( SigmoidFilter ),
			   &FilterObject::database );
  
  namespace SigmoidFilterInternals {
    FIELDDB_ELEMENT( SigmoidFilter, inputIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( SigmoidFilter, alpha, INPUT_OUTPUT );
    FIELDDB_ELEMENT( SigmoidFilter, beta, INPUT_OUTPUT );
    
    template<class InputImageType, class OutputImageType>
    void executeFilter(InputImageType *in,
		       OutputImageType* &out,
		       H3DFloat alpha,
		       H3DFloat beta) {
      
      wish::SigmoidFilter<InputImageType, OutputImageType> sf;
      sf.setInput( in );
      sf.setNormalizedAlpha( alpha );
      sf.setNormalizedBeta( beta );
      sf.setNormalizedMax( 1.0f );
      sf.setNormalizedMin( 0.0f );
      cerr << "Executing SigmoidFilter..." << endl; 
      sf.execute();
      cerr << "Execution time = " << sf.getProcessTime() << " s" << endl;
      
      out = sf.releaseOutput();
    }
    
  }
  
  SigmoidFilter::SigmoidFilter() :
    inputIndex( new SFInt32 ),
    alpha( new SFFloat ),
    beta( new SFFloat )
  {  
    this->type_name = "SigmoidFilter";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // defaults
    inputIndex->setValue( -1 );
    alpha->setValue(1.0f);
    beta->setValue(10.0f);
    
    WISH_H3D_CREATE_STOP();
    
  }
  
  wish::ImageObject* SigmoidFilter::execute() {
    Volume *vol = volume->getValue();
    if( !vol ) 
      return 0;
    
    wish::ImageObject *input = vol->getImage( inputIndex->getValue() );
    if( !input )
      return 0;
    
    string dt = input->getDataType();
    if( dt=="uint8" ) {
      wish::Image8 *in = dynamic_cast<wish::Image8*>(input);
      wish::Image8 *out;
      SigmoidFilterInternals::
	executeFilter( in, out, 
		       alpha->getValue(),
		       beta->getValue() );
      return out;
    }
    else if( dt=="uint16" ) {
      wish::Image16 *in = dynamic_cast<wish::Image16*>(input);
      wish::Image16 *out;
      SigmoidFilterInternals::
	executeFilter( in, out, 
		       alpha->getValue(),
		       beta->getValue() );
      return out;
    }
    else if( dt=="float" ) {
      wish::Imagef *in = dynamic_cast<wish::Imagef*>(input);
      wish::Imagef *out;
      SigmoidFilterInternals::
	executeFilter( in, out, 
		       alpha->getValue(),
		       beta->getValue() );
      return out;
    }
    else {
      return 0;
    }
    
  }
  
}
