///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#include "wishH3DThresholdFilter.hh"

#include "wishThresholdFilter.hh"

namespace wishH3D {
    
  // Add this node to the H3DNodeDatabase system.
  H3DNodeDatabase 
  ThresholdFilter::database( "wishH3DThresholdFilter", 
			     &(newInstance<ThresholdFilter>), 
			     typeid( ThresholdFilter ),
			     &FilterObject::database );
  
  namespace ThresholdFilterInternals {
    FIELDDB_ELEMENT( ThresholdFilter, inputIndex, INPUT_OUTPUT );
    FIELDDB_ELEMENT( ThresholdFilter, lowThreshold, INPUT_OUTPUT );
    FIELDDB_ELEMENT( ThresholdFilter, highThreshold, INPUT_OUTPUT );
    
    template<class InputImageType, class OutputImageType>
    void executeFilter(InputImageType *in,
		       OutputImageType* &out,
		       H3DFloat lowthreshold,
		       H3DFloat highthreshold) {
      
      wish::ThresholdFilter<InputImageType, OutputImageType> tf;
      tf.setInput( in );
      tf.thresholdBetween( lowthreshold, highthreshold );
      cerr << "Executing ThresholdFilter..." << endl; 
      tf.execute();
      cerr << "Execution time = " << tf.getProcessTime() << " s" << endl;
      
      out = tf.releaseOutput();
    }
    
  }
  
  ThresholdFilter::ThresholdFilter() :
    inputIndex( new SFInt32 ),
    lowThreshold( new SFFloat ),
    highThreshold( new SFFloat )
  {  
    this->type_name = "ThresholdFilter";
    database.initFields( this );
    
    WISH_H3D_CREATE_START();
    
    // defaults
    inputIndex->setValue( -1 );
    lowThreshold->setValue( 0.0f );
    highThreshold->setValue( 255.0f );
    
    WISH_H3D_CREATE_STOP();
    
  }
  
  wish::ImageObject* ThresholdFilter::execute() {
    Volume *vol = volume->getValue();
    if( !vol ) 
      return 0;
    
    wish::ImageObject *input = vol->getImage( inputIndex->getValue() );
    if( !input )
      return 0;
    
    // output
    wish::Image8 *out = 0;
    
    string dt = input->getDataType();
    if( dt=="uint8" ) {
      ThresholdFilterInternals::
	executeFilter( dynamic_cast<wish::Image8*>(input),
		       out,
		       lowThreshold->getValue(),
		       highThreshold->getValue() );
    }
    else if( dt=="uint16" ) {
      ThresholdFilterInternals::
	executeFilter( dynamic_cast<wish::Image16*>(input),
		       out,   
		       lowThreshold->getValue(),
		       highThreshold->getValue() );
    }
    else if( dt=="float" ) {
      ThresholdFilterInternals::
	executeFilter( dynamic_cast<wish::Imagef*>(input),
		       out,   
		       lowThreshold->getValue(),
		       highThreshold->getValue() );
    }
    
    return out;
    
  }
  
}
