///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __WISH_H3D_TRANSFERFUNCTIONTEXTURE_HH__
#define __WISH_H3D_TRANSFERFUNCTIONTEXTURE_HH__

#include "X3DTexture2DNode.h"
#include "PixelTexture.h"
#include "PixelImage.h"

#include "wishH3DTransferFunction.hh"
#include "wishH3DColorTransferFunction.hh"
#include "wishH3D.hh"

using namespace H3D;

namespace wishH3D {
  
  class WISHH3D_API TransferFunctionTexture : public PixelTexture {
  public:
    static const int NTEX;
    
    // interface
    static H3DNodeDatabase database;
    
    /// field that updates the texture image from the transfer functions
    struct UpdateImage : 
      public AutoUpdate< TypedField<Field, void, 
                         Types<SFTransferFunction, 
                               SFTransferFunction, 
                               SFTransferFunction, 
                               SFTransferFunction> > > {
      virtual void update();
    };  
    
    // x3d 
    virtual string defaultXMLContainerField() {
      return "texture";
    }
    
    //! constructor
    TransferFunctionTexture();
    
    //! image updater
    auto_ptr<UpdateImage> updateImage;
    
    //! transfer functions
    auto_ptr<SFTransferFunction> red;
    auto_ptr<SFTransferFunction> green;
    auto_ptr<SFTransferFunction> blue;
    auto_ptr<SFTransferFunction> alpha;
  };
  
}

#endif
