///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __WISH_H3D_VOLUME_OUTLINE_HH__
#define __WISH_H3D_VOLUME_OUTLINE_HH__

#include "Shape.h"

#include "wishH3DVolume.hh"
#include "wishH3DVolumeGraphics.hh"

using namespace H3D;

namespace wishH3D {
  
  // simple geometry that draws an outline
  class WISHH3D_API VolumeOutlineGeometry : public X3DGeometryNode {
  public:
    
    typedef X3DGeometryNode BaseClass;

    /// SFBound is specialized update itself from the total extent
    /// routes_in[0] = total extent
    class SFBound : public TypedField< X3DGeometryNode::SFBound, SFVec3f > {
      virtual void update() {
	Vec3f te = static_cast<SFVec3f*>( routes_in[0] )->getValue();
	BoxBound *bb = new BoxBound;
	bb->size->setValue( 1.1*te );
	value = bb;
      }
    };
    
    VolumeOutlineGeometry( Inst< SFNode > _metadata = 0,
			   Inst< SFBound > _bound = 0,
			   Inst< SFVec3f > _totalExtent = 0 );
    
    virtual void initialize();
    
    virtual void render();
    
    auto_ptr<SFVec3f> totalExtent;
   
  };
  
  class WISHH3D_API VolumeOutline : public VolumeGraphics {
  public:
    
    typedef VolumeGraphics BaseClass;
    
    /// Constructor.
    VolumeOutline();
    
    /// initialize
    virtual void initialize();
    
    /// dimensions and spacing 
    auto_ptr< SFVolume > volume;
    auto_ptr< SFColor > color;
    auto_ptr< SFFloat > thickness;
    
    /// The H3DNodeDatabase for this node.
    static H3DNodeDatabase database;
    
  protected:
  };
}

#endif
