///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_VOLUMERENDERER_HH_
#define _WISH_H3D_VOLUMERENDERER_HH_

#include "X3DChildNode.h"
#include "X3DBoundedObject.h"
#include "H3DDisplayListObject.h"
#include "X3DTexture3DNode.h"
#include "X3DTexture2DNode.h"

#include "ComposedShader.h"
#include "ShaderPart.h"

#include "DependentNodeFields.h"
#include "PeriodicUpdate.h"
#include "SFFloat.h"
#include "SFBool.h"
#include "SFColor.h"
#include "SFVec3f.h"
#include "SFMatrix4f.h"

#include "wishH3DVolume.hh"

namespace wishH3D {
  
  // VolumeRenderer
  class WISHH3D_API VolumeRenderer : public Group {
  public:
    typedef Group BaseClass;
    
    /// Field that updates texture matrices from the volume
    struct UpdateDimensions : public PeriodicUpdate<SFVec3f> {
      virtual void update();
    };
    
    /// Bound is updated from the total extent field
    struct SFBound : public TypedField<H3DBoundedObject::SFBound, SFVec3f> {
      virtual void update();
    };
    
    typedef DependentSFNode<X3DTexture3DNode, 
			    FieldRef<H3DDisplayListObject,
				     H3DDisplayListObject::DisplayList,
				     &H3D::H3DDisplayListObject::displayList>, 
			    true > SFTexture3DNode;
    
    typedef DependentSFNode<X3DTexture2DNode, 
			    FieldRef<H3DDisplayListObject,
				     H3DDisplayListObject::DisplayList,
				     &H3D::H3DDisplayListObject::displayList>, 
			    true > SFTexture2DNode;
    
    //! constructor
    VolumeRenderer();
    
    //! initializing function
    virtual void initialize();
    
    //! render functions
    virtual void render();
    void renderBox(const Vec3f& te) const;
    
    //! traverse the scenegraph
    virtual void traverseSG( TraverseInfo &ti );
    
    //! the volume
    auto_ptr<SFVolume> volume;
    
    //! the texture
    auto_ptr<SFTexture3DNode> texture;
    
    //! the normal texture
    auto_ptr<SFTexture3DNode> normalTexture;
    auto_ptr<SFFloat> isoValue;
    
    //! transfer function texture
    auto_ptr<SFTexture2DNode> tfTexture;
    
    //! render mode
    auto_ptr<SFInt32> renderMode;
    
    //! active
    auto_ptr<SFBool> active;

    //! the shaders
    auto_ptr<ComposedShader> shader;
    
    //! rayStep
    auto_ptr<SFFloat> rayStep;
    
    //! texture matrices
    auto_ptr<SFMatrix4f> textureMatrix; // world -> texture
    auto_ptr<SFMatrix4f> textureMatrixInverse; // texture -> world
    
    //! update texture matrices
    auto_ptr<UpdateDimensions> updateDimensions;
    
    // interface
    static H3DNodeDatabase database;
    
  };
  
}

#endif
