///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_VOLUME_TEXTURE_HH_
#define _WISH_H3D_VOLUME_TEXTURE_HH_

#include "PeriodicUpdate.h"
#include "X3DTexture3DNode.h"

#include "wishH3DVolumeImage.hh"
#include "wishH3DVolume.hh"
#include "wishH3DVolumeChild.hh"

namespace wishH3D {
  
  class WISHH3D_API VolumeTexture : public X3DTexture3DNode, 
				    public VolumeChild {
  public:
    
    typedef X3DTexture3DNode BaseClass;
    
    // to this one we can route an integer field to change the image
    // used by this VolumeTexture instance
    struct ImageChanger : public PeriodicUpdate<SFInt32> {
      virtual void update();
    };
    
    // interface
    static H3DNodeDatabase database;
    
    // constructor
    VolumeTexture();
    
    // get the image
    wish::ImageObject* getImage();
    
    // x3d 
    virtual string defaultXMLContainerField() {
      return "texture";
    }
    
    //! initializing function
    virtual void initialize();
    
    //! generate the texture from the imageIndex field
    virtual void generateTexture();
    
    //! imageIndex
    auto_ptr<SFInt32> imageIndex;
    
    //! image changer
    auto_ptr<ImageChanger> imageChanger;
    
  protected:
    string interpolate_string;
  };
  
}

#endif
