///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007, Erik Vidholm
//
//
//   This file is part of WISH.
//
//   WISH is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   WISH is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with WISH.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _WISH_H3D_VOLUME_TEXTURE_GRAPHICS_HH_
#define _WISH_H3D_VOLUME_TEXTURE_GRAPHICS_HH_

#include "SFNode.h"
#include "MFNode.h"
#include "SFInt32.h"
#include "DependentNodeFields.h"

#include "wishH3DSFTFTexture.hh"
#include "wishH3DVolumeTexture.hh"
#include "wishH3DVolumeGraphics.hh"

namespace wishH3D {
  
  typedef TypedSFNode<X3DTexture3DNode> SFX3DTexture3DNode;
  
  // VolumeTextureGraphics is a base class for graphical 
  // display of volume images using 3D textures
  class WISHH3D_API VolumeTextureGraphics : public VolumeGraphics {
  public:
    typedef VolumeGraphics BaseClass;
    
    /// Field that updates texture matrices from the volume
    struct UpdateDimensions : public PeriodicUpdate<SFVec3f> {
      virtual void update();
    };
    
    // make the volume texture a dependent node by letting the change 
    // of imageIndex generate an event 
    typedef 
    DependentSFNode< FieldRef<VolumeTexture,
			      SFInt32,
			      &VolumeTexture::imageIndex> > SFVolumeTexture;
    
    // interface
    static H3DNodeDatabase database;
    
    // constructor
    VolumeTextureGraphics();
    
    //! initializing function
    virtual void initialize();
    
    //! the texture
    auto_ptr<SFVolumeTexture> texture;
    auto_ptr<SFVolumeTexture> overlay;
    
    //! window/level contrast adjustment
    auto_ptr<SFFloat> window;
    auto_ptr<SFFloat> level;
    
    //! the component 0-3 where the intensity is stored 
    auto_ptr<SFInt32> intensityComponent;
    
    //! transfer function
    auto_ptr<SFTransferFunction> transferFunction;
    
    //! the texture representation of the transfer function
    auto_ptr<SFTFTexture> tfTexture;
    
    //! texture matrices
    auto_ptr<SFMatrix4f> textureMatrix; // world -> texture
    auto_ptr<SFMatrix4f> textureMatrixInverse; // texture -> world
    
    //! update dimensions
    auto_ptr<UpdateDimensions> updateDimensions;
    
  };
  
}

#endif
